Package: GBASS
Title: Generalized Bayesian Adaptive Smoothing Splines
Version: 2.0.1
Authors@R: 
    person(given = "Kellin",
           family = "Rumsey",
           role = c("aut", "cre"),
           email = "knrumsey@lanl.gov")
Description: Bayesian nonlinear regression under a range of likelihood models
    using generalized Bayesian adaptive smoothing splines. Robust regression
    with Student's t likelihoods, quantile regression, and related latent-scale
    models are included as special cases.
License: BSD_3_clause + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
Depends: R (>= 3.5.0)
Imports: Matrix, GIGrvg, BASS
Suggests: knitr, rmarkdown, lhs, testthat (>= 2.1.0)
NeedsCompilation: no
Packaged: 2026-04-22 17:02:42 UTC; knrumsey
Author: Kellin Rumsey [aut, cre]
Maintainer: Kellin Rumsey <knrumsey@lanl.gov>
Repository: CRAN
Date/Publication: 2026-04-24 20:10:16 UTC
Built: R 4.5.2; ; 2026-04-24 23:26:07 UTC; unix

Package: GBASS
Title: Generalized Bayesian Adaptive Smoothing Splines
Version: 2.0.1
Authors@R: 
    person(given = "Kellin",
           family = "Rumsey",
           role = c("aut", "cre"),
           email = "knrumsey@lanl.gov")
Description: Bayesian nonlinear regression under a range of likelihood models
    using generalized Bayesian adaptive smoothing splines. Robust regression
    with Student's t likelihoods, quantile regression, and related latent-scale
    models are included as special cases.
License: BSD_3_clause + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
Depends: R (>= 3.5.0)
Imports: Matrix, GIGrvg, BASS
Suggests: knitr, rmarkdown, lhs, testthat (>= 2.1.0)
NeedsCompilation: no
Packaged: 2026-04-22 17:02:42 UTC; knrumsey
Author: Kellin Rumsey [aut, cre]
Maintainer: Kellin Rumsey <knrumsey@lanl.gov>
Repository: CRAN
Date/Publication: 2026-04-24 20:10:16 UTC
Built: R 4.6.0; ; 2026-04-26 03:43:13 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,gbass)
S3method(predict,gbass)
S3method(print,gbass)
export(build_GBP)
export(build_GIG)
export(build_prior)
export(gbass)
export(gbass2bass)
export(gm2bm)
export(gsobol)
export(hbass)
export(nw_est_mom)
export(nw_gamma_prior)
export(nw_triangle)
export(nwbass)
export(nwbass2)
export(qbass)
export(rgig2)
export(rmhn)
export(tbass)
import(Matrix)
importFrom(grDevices,adjustcolor)
importFrom(graphics,curve)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(stats,dnorm)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,rbinom)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(utils,combn)

#' Generalized Inverse Gaussian Generator
#'
#' This function generates samples from the GIG(p, a, b) distribution with density function
#' f(x) = cx^(p-1)exp(-1/2*(a*x + b/x))
#'
#' @param p a real valued parameter
#' @param a a non-negative parameter. If a=0, then p must be negative.
#' @param b a non-negative parameter. If b=0, then p must be positive.
#' @details A uniformly bounded rejection sample based on Hörmann et. al. (2014). Special cases include Gamma (b=0, p>0), Inverse Gamma (a=0, p<0) and Inverse Gaussian (p=-1/2).
#' @return A numeric vector of random draws from the generalized inverse Gaussian distribution.
#' @references Hörmann, Wolfgang, and Josef Leydold. "Generating generalized inverse Gaussian random variates." Statistics and Computing 24.4 (2014): 547-557.
#' @export
#' @examples
#' x <- rep(NA, 1000)
#' for(i in 1:1000) x[i] <- rgig2(2, 3, 0.5)
#' hist(x)
#'
rgig2 <- function(p, a, b){
  res <- ifelse(a==0, 1/rgamma(1, -p, b/2), GIGrvg::rgig(1, lambda=p, psi=a, chi=b))
}
rgig2.vec <- Vectorize(rgig2, "b")
mu_gig <- function(p, a, b){
  # Inverse gamma case
  if(a == 0){
    if(p >= 0){
      stop("Invalid parameters for GIG. When a=0, p must be negative")
    }
    mu <- -b/(2*(p+1))
  }
  # Gamma case
  if(b == 0){
    if(p <= 0){
      stop("Invalid parameters for GIG. When b=0, p must be positive")
    }
    mu <- 2*p/a
  }
  if(a != 0 & b != 0){
    mu <- sqrt(b/a)*besselK(sqrt(a*b), p+1)/besselK(sqrt(a*b), p)
  }
  return(mu)
}
rgig2.vec_fh <- Vectorize(rgig2, c("p", "a"))
var_gig <- function(p, a, b){
  # Inverse gamma case
  if(a == 0){
    if(p >= 0){
      stop("Invalid parameters for GIG. When a=0, p must be negative")
    }
    sig2 <- -b^2/(4*(p+1)^2*(p+2))
  }
  # Gamma case
  if(b == 0){
    if(p <= 0){
      stop("Invalid parameters for GIG. When b=0, p must be positive")
    }
    sig2 <- 4*p/a^2
  }
  if(a != 0 & b != 0){
    sig2 = (b/a)*(besselK(sqrt(a*b), p+2)/besselK(sqrt(a*b), p) - (besselK(sqrt(a*b), p+1)/besselK(sqrt(a*b), p))^2)
  }
  return(sig2)
}

#' Convert GBASS object to BASS-like object
#'
#' Converts an object of class \code{gbass} to an object with class \code{bass},
#' so that downstream BASS utilities such as Sobol decomposition can be used.
#'
#' @param gm An object of class \code{gbass}.
#'
#' @return An object with class \code{c("bass", "gbass")}.
#' @export
gbass2bass <- function(gm) {
  if (!inherits(gm, "gbass")) {
    stop("gm must be an object of class 'gbass'")
  }

  needed <- c("a", "M", "X", "basis", "lookup")
  missing_fields <- needed[!needed %in% names(gm)]
  if (length(missing_fields) > 0) {
    stop("gbass object is missing required fields: ",
         paste(missing_fields, collapse = ", "))
  }

  out <- list()
  out$degree <- 1

  nmcmc <- length(gm$a)
  maxb <- max(gm$M)

  beta <- matrix(NA_real_, nrow = nmcmc, ncol = maxb + 1)
  for (i in seq_len(nmcmc)) {
    beta[i, seq_along(gm$a[[i]])] <- gm$a[[i]]
  }

  out$beta <- beta
  out$nbasis <- gm$M
  out$p <- ncol(gm$X)

  model.lookup <- integer(nmcmc)
  model.lookup[1] <- 1L
  for (i in 2:nmcmc) {
    same_model <- gm$M[i] == gm$M[i - 1] &&
      identical(gm$basis[[i]], gm$basis[[i - 1]])

    model.lookup[i] <- if (same_model) model.lookup[i - 1] else model.lookup[i - 1] + 1L
  }

  out$model.lookup <- model.lookup
  out$n.models <- max(model.lookup)

  out$des <- TRUE
  out$func <- FALSE
  out$cat <- FALSE

  max.int <- max(vapply(gm$lookup, function(zz) length(zz$s), integer(1)))
  n.int.des <- matrix(0, nrow = out$n.models, ncol = maxb)
  signs.des <- vars.des <- knots.des <- array(NA_real_, dim = c(out$n.models, maxb, max.int))

  for (i in seq_len(out$n.models)) {
    ind <- which(model.lookup == i)
    draw_idx <- ind[1]

    if (gm$M[draw_idx] > 0) {
      for (j in seq_len(gm$M[draw_idx])) {
        basis_id <- gm$basis[[draw_idx]][j]
        basis_obj <- gm$lookup[[basis_id]]

        n.int.des[i, j] <- basis_obj$J
        signs.des[i, j, seq_len(basis_obj$J)] <- basis_obj$s
        vars.des[i, j, seq_len(basis_obj$J)]  <- basis_obj$u
        knots.des[i, j, seq_len(basis_obj$J)] <- basis_obj$t
      }
    }
  }

  out$xx.des <- gm$X
  out$n.int.des <- n.int.des
  out$signs.des <- signs.des
  out$knots.des <- knots.des
  out$vars.des <- vars.des
  out$cx <- rep("numeric", out$p)
  out$range.des <- rbind(apply(gm$X, 2, min), apply(gm$X, 2, max))

  out$nburn <- 0
  out$thin <- 1
  out$nmcmc <- nmcmc
  out$pdes <- out$p
  out$pcat <- 0
  out$pfunc <- 0
  out$maxInt.des <- max.int
  out$maxInt.cat <- 0
  out$maxInt.func <- 0

  class(out) <- c("bass", class(gm))
  out
}

#' @rdname gbass2bass
#' @export
gm2bm <- gbass2bass

dmwnchBass<-function(z.vec,vars){
  z.vec <- z.vec/sum(z.vec)
  alpha<-z.vec[vars]/sum(z.vec[-vars])
  j<-length(alpha)
  ss<-1 + (-1)^j * 1/(sum(alpha)+1)
  if(j > 1){
    for(i in 1:(j-1))
      ss <- ss + (-1)^(i) * sum(1/(colSums(combn(alpha,i))+1))
  }
  ss
}

logdet <- function(A) Matrix::determinant(A)$modulus
move_type <- function(M, Mmax, Pmove){
  if(M <= 0)    return("B")
  if(M >= Mmax) return(sample(c("D", "M"), 1, prob=Pmove[2:3]))
  return(sample(c("B", "D", "M"), 1, prob=Pmove))
}
rgbp<-function(v_curr,a, b, p, prop_sigma,lb, w_curr,c,r,beta_curr){
  v_cand <- exp(log(v_curr) + rnorm(1, 0, prop_sigma))
  if(v_cand >= lb){
    alpha_v <- (p*a - 1/2)*(log(v_cand) - log(v_curr)) -
      r^2/(2*w_curr*c)*(1/v_cand - 1/v_curr) -
      (a + b)*(log(1+v_cand^p) - log(1+v_curr^p)) -
      beta_curr^2/(2*c)*(v_cand - v_curr)
  }else{
    alpha_v <- -Inf
  }
  if(log(runif(1)) < alpha_v){
    return(v_cand)
  }
  return(v_curr)
}
rgbp.vec<-Vectorize(rgbp,c("v_curr","r"))
TCP  <- function(...) Matrix::tcrossprod(...)
symchol <- function(...) base::chol(...)

myTimestamp <-function(){
  x<-Sys.time()
  paste('#--',format(x,"%b %d %X"),'--#')
}

## makes negative values 0
pos<-function(vec){
  #replace(vec,vec<0,0)
  (abs(vec)+vec)/2
}

## largest value of basis function, assuming x's in [0,1], used for scaling
const<-function(signs,knots,degree){
  cc<-prod(((signs+1)/2 - signs*knots))^degree
  if(cc==0)
    return(1)
  return(cc)
} # since a product, can find for functional & categorical pieces separately, take product

## make basis function (from continuous variables)
makeBasis<-function(signs,vars,knots,datat,degree){
  cc<-const(signs,knots,degree)
  temp1<-pos(signs*(datat[vars,,drop=F]-knots))^degree # this only works for t(data)...
  if(length(vars)==1){
    return(c(temp1)/cc)
  } else{
    temp2<-1
    for(pp in 1:length(vars)){ # faster than apply
      temp2<-temp2*temp1[pp,]
    }
    return(temp2/cc)
  }
}

#' @importFrom grDevices adjustcolor
#' @importFrom graphics curve hist legend par points segments
#' @importFrom stats dnorm optim pnorm predict rbinom rgamma rnorm runif sd var
#' @importFrom utils combn
"_PACKAGE"

#' Sobol decomposition for GBASS models
#'
#' Computes Sobol sensitivity indices for a fitted \code{gbass} model by first
#' converting it to a compatible \code{BASS} object with \code{gbass2bass()},
#' and then calling \code{BASS::sobol()}.
#'
#' @param object A fitted object of class \code{"gbass"}.
#' @param ... Additional arguments passed to \code{BASS::sobol()}.
#'
#' @return An object of class \code{"bassSob"} returned by \code{BASS::sobol()}.
#'
#' @details
#' This is a thin wrapper around \code{BASS::sobol()} for GBASS models.
#' Users who want direct access to the converted \code{BASS} object can call
#' \code{gbass2bass()} explicitly and then use \code{BASS::sobol()} themselves.
#'
#' @seealso \code{\link{gbass2bass}}, \code{\link[BASS]{sobol}}
#'
#' @examples
#' ff1 <- function(x) 10.391*((x[1]-0.4)*(x[2]-0.6) + 0.36)
#'
#' n <- 100
#' p <- 4
#' X <- matrix(runif(n * p), nrow = n)
#' y <- apply(X, 1, ff1)
#'
#' mod <- gbass(X, y, nmcmc = 1000, nburn = 900)
#'
#' # Direct wrapper
#' sob <- gsobol(mod)
#'
#' # Equivalent manual conversion
#' bm <- gbass2bass(mod)
#' sob2 <- BASS::sobol(bm)
#'
#' @export
gsobol <- function(object, ...) {
  BASS::sobol(gbass2bass(object), ...)
}
gsobol <- function(object, ...) {
  BASS::sobol(gbass2bass(object), ...)
}

#' Generalized Bayesian MARS with a Normal-Wald likelihood
#'
#' Fits a generalized BMARS model under a Normal-Wald likelihood. This provides
#' flexible nonlinear regression with a unimodal, potentially skewed error model.
#'
#' @param X An \eqn{N \times p} numeric matrix of predictor variables. A numeric
#'   vector is treated as a single-column matrix.
#' @param y A numeric response vector of length \eqn{N}.
#' @param w_prior A named list specifying the prior for the global variance
#'   component. See Details.
#' @param maxInt Integer giving the maximum degree of interaction in spline basis
#'   functions. Defaults to \code{3}.
#' @param maxBasis Maximum number of basis functions.
#' @param npart Minimum number of nonzero points required for a proposed basis
#'   function. Defaults to \code{min(20, 0.1 * N)}.
#' @param nmcmc Total number of MCMC iterations.
#' @param nburn Number of initial MCMC iterations discarded as burn-in.
#' @param thin Thinning interval for retained draws.
#' @param moveProbs A length-3 vector giving probabilities for birth, death, and
#'   mutation moves.
#' @param a_tau Prior hyperparameter for \code{tau}.
#' @param b_tau Prior hyperparameter for \code{tau}. Defaults to \code{N / 2}.
#' @param a_lambda Prior hyperparameter for \code{lambda}.
#' @param b_lambda Prior hyperparameter for \code{lambda}.
#' @param m_beta Prior mean for \code{beta}.
#' @param s_beta Prior standard deviation for \code{beta}.
#' @param lag_beta Number of initial iterations for which \code{beta} is fixed at
#'   \code{m_beta}. This is often used to stabilize early sampling.
#' @param m_gamma Prior mean for \code{gamma}.
#' @param s_gamma Prior standard deviation for \code{gamma}.
#' @param scale Fixed variance-scale parameter. Defaults to \code{1}.
#' @param Iw0 Vector of positive nominal weights for interaction order in proposed
#'   basis functions. Must have length \code{maxInt}.
#' @param Zw0 Vector of positive nominal weights for variable selection in proposed
#'   basis functions. Must have length \code{ncol(X)}.
#' @param verbose Logical; should progress be printed?
#'
#' @details
#' The latent local-scale prior is fixed internally to the Normal-Wald form
#' \eqn{v_i \sim \mathrm{GIG}(-1/2, \gamma^2, 1)}. Unlike \code{gbass()},
#' \code{nwbass()} does not expose a user-specified \code{v_prior}.
#'
#' The \code{w_prior} list should contain:
#' \enumerate{
#'   \item \code{type}: either \code{"GIG"} or \code{"GBP"}.
#'   \item \code{p}, \code{a}, \code{b}: hyperparameters for the prior.
#'   \item \code{lower_bound}: optional lower bound for \code{w}. For backward
#'     compatibility, \code{lb} is also accepted.
#'   \item \code{prop_sigma}: proposal standard deviation on the log scale for
#'     Metropolis updates of \code{w}. This is only used when \code{w} is updated
#'     by Metropolis-Hastings, such as the \code{"GBP"} case.
#'   \item \code{adapt}, \code{adapt_delay}, \code{adapt_thin}: optional controls
#'     for adaptive Metropolis updates of \code{w} when applicable.
#' }
#'
#' Retained draws are taken at iterations
#' \code{nburn + 1, nburn + 1 + thin, ...}. Thus \code{nburn} is interpreted as
#' the number of initial iterations discarded as burn-in.
#'
#' @return
#' An object of class \code{c("nwbass", "gbass")} containing posterior draws and
#' fitted model information.
#'
#' @import Matrix
#' @export
#'
#' @examples
#' \donttest{
#' n <- 200
#' X <- lhs::maximinLHS(n, 2)
#' f <- 20 * apply(X, 1, function(x) sin(pi * x[1]) + x[2]^2)
#' eps <- rgamma(n, 3, 1.5) - 2
#' y <- f + eps
#'
#' mod <- nwbass(X, y,
#'               m_beta=0, s_beta=10,
#'               m_gamma = nw_gamma_prior()[1], s_gamma = nw_gamma_prior()[2])
#' }
nwbass <- function(
    X, y,
    w_prior = list(type = "GIG", p = -0.1, a = 0, b = 0.1),
    maxInt = 3, maxBasis = 1000, npart = NULL,
    nmcmc = 10000, nburn = 9000, thin = 1,
    moveProbs = rep(1 / 3, 3),
    a_tau = 1 / 2, b_tau = NULL,
    a_lambda = 1, b_lambda = 1,
    m_beta = 0, s_beta = 0, lag_beta = 1001,
    m_gamma = 1, s_gamma = 0,
    scale = 1,
    Iw0 = rep(1, maxInt), Zw0 = rep(1, ncol(X)),
    verbose = TRUE
) {
  # ---------------------------------------------------------------------------
  # Input checks and normalization
  # ---------------------------------------------------------------------------
  if (is.null(dim(X))) {
    X <- matrix(X, ncol = 1)
  } else {
    X <- as.matrix(X)
  }

  if (!is.numeric(X)) {
    stop("X must be numeric.")
  }
  if (!is.numeric(y)) {
    stop("y must be numeric.")
  }
  if (anyNA(X) || anyNA(y)) {
    stop("X and y must not contain missing values.")
  }
  if (nrow(X) != length(y)) {
    stop("nrow(X) and length(y) should match.")
  }
  if (!is.numeric(nmcmc) || length(nmcmc) != 1L || nmcmc < 1 || nmcmc != as.integer(nmcmc)) {
    stop("nmcmc must be a positive integer.")
  }
  if (!is.numeric(nburn) || length(nburn) != 1L || nburn < 0 || nburn != as.integer(nburn)) {
    stop("nburn must be a nonnegative integer.")
  }
  if (nburn >= nmcmc) {
    stop("nburn must be strictly less than nmcmc.")
  }
  if (!is.numeric(thin) || length(thin) != 1L || thin < 1 || thin != as.integer(thin)) {
    stop("thin must be a positive integer.")
  }
  if (!is.numeric(maxBasis) || length(maxBasis) != 1L || maxBasis < 1 || maxBasis != as.integer(maxBasis)) {
    stop("maxBasis must be a positive integer.")
  }
  if (!is.numeric(maxInt) || length(maxInt) != 1L || maxInt < 1 || maxInt != as.integer(maxInt)) {
    stop("maxInt must be a positive integer.")
  }
  if (!is.numeric(scale) || length(scale) != 1L || scale <= 0) {
    stop("scale must be a positive numeric scalar.")
  }
  if (!is.numeric(moveProbs) || length(moveProbs) != 3L || any(moveProbs < 0) || sum(moveProbs) <= 0) {
    stop("moveProbs must be a numeric vector of length 3 with nonnegative entries and positive sum.")
  }
  moveProbs <- moveProbs / sum(moveProbs)

  N <- nrow(X)
  p <- ncol(X)
  maxInt <- min(maxInt, p)

  if (length(Iw0) != maxInt || any(Iw0 <= 0)) {
    stop("Iw0 must have length maxInt and positive entries.")
  }
  if (length(Zw0) != p || any(Zw0 <= 0)) {
    stop("Zw0 must have length ncol(X) and positive entries.")
  }

  if (!is.numeric(lag_beta) || length(lag_beta) != 1L || lag_beta < 0 || lag_beta != as.integer(lag_beta)) {
    stop("lag_beta must be a nonnegative integer.")
  }
  if (lag_beta > nburn) {
    lag_beta <- nburn
    warning("lag_beta cannot exceed nburn. Setting lag_beta = nburn.")
  }

  if (is.null(w_prior$type) || !(w_prior$type %in% c("GIG", "GBP"))) {
    stop("w_prior$type must be either 'GIG' or 'GBP'.")
  }
  if (!is.null(w_prior$lb) && is.null(w_prior$lower_bound)) {
    w_prior$lower_bound <- w_prior$lb
  }

  if (is.null(npart)) {
    npart <- min(20, 0.1 * N)
  }
  if (is.null(b_tau)) {
    b_tau <- N / 2
  }

  keep_idx <- seq.int(from = nburn + 1L, to = nmcmc, by = thin)
  nkeep <- length(keep_idx)
  keep_iter <- rep(FALSE, nmcmc)
  keep_iter[keep_idx] <- TRUE

  # Fixed latent-scale prior for the Normal-Wald model
  v_prior <- list(type = "GIG", p = -1 / 2, a = 1, b = 1)

  # Initial moments under the current NW latent structure
  mu_v <- 1
  s2_v <- 1

  if (is.null(w_prior$lower_bound)) {
    w_prior$lower_bound <- var(y) / N
  }

  # ---------------------------------------------------------------------------
  # Allocate storage
  # ---------------------------------------------------------------------------
  a_mc <- list()
  M_mc <- lam_mc <- tau_mc <- w_mc <- bet_mc <- gam_mc <- ss <- bias_mc <- s2_mc <- rep(NA_real_, nkeep)
  v_mc <- matrix(NA_real_, nrow = nkeep, ncol = N)
  lookup <- basis_mc <- list()
  basis_index <- integer(0)
  nlook <- 1L
  kk <- 1L

  # ---------------------------------------------------------------------------
  # Initialize parameters
  # ---------------------------------------------------------------------------
  M <- 0L
  tau <- (b_tau + 0.1) / (a_tau + 0.1)
  gam <- 1
  v <- rep(1, N)
  w <- var(y) / scale
  lam <- rgamma(1, a_lambda, b_lambda)
  bet <- rnorm(1, m_beta, s_beta)
  a <- mean(y)

  s_beta_hold <- s_beta
  if (lag_beta > 0L) {
    s_beta <- 0
  }

  z <- y - bet * v * sqrt(w)
  B <- matrix(1, nrow = N)
  Vinv <- Matrix::Diagonal(x = 1 / v)
  U <- solve(symchol(crossprod(B, Vinv) %*% B + scale / tau * Diagonal(M + 1L)))
  U2 <- crossprod(z / v, B) %*% U

  bias <- sqrt(w) * bet * mu_v
  s2 <- scale * w * mu_v + w * bet^2 * s2_v

  cnt1 <- cnt2 <- rep(0, 3)
  cntw <- 0
  adapt_iter <- rep(FALSE, nmcmc)

  if (w_prior$type == "GBP") {
    if (is.null(w_prior$prop_sigma)) {
      w_prior$prop_sigma <- min(max(var(y) / scale / 2, 1e-3), 5)
    }
    if (is.null(w_prior$adapt)) {
      w_prior$adapt <- TRUE
    }

    if (w_prior$adapt) {
      if (is.null(w_prior$adapt_delay)) {
        w_prior$adapt_delay <- floor(nburn / 10)
      }
      if (is.null(w_prior$adapt_thin)) {
        w_prior$adapt_thin <- 1L
      }

      w_prior$count <- 1L
      w_prior$sum1 <- log(w)
      w_prior$welford <- 0
      adapt_iter <- seq_len(nmcmc) %in% seq.int(1L, nburn, by = w_prior$adapt_thin)
    }
  }

  tX <- t(X)

  if (verbose) {
    pr <- c("MCMC iteration", 0, myTimestamp(), "nbasis:", M)
    cat(pr, "\n")
  }

  # ---------------------------------------------------------------------------
  # MCMC loop
  # ---------------------------------------------------------------------------
  for (k in seq_len(nmcmc)) {
    if (k == lag_beta) {
      s_beta <- s_beta_hold
    }

    move <- move_type(M, maxBasis, moveProbs)

    # -------------------------------------------------------------------------
    # Birth step
    # -------------------------------------------------------------------------
    if (move == "B") {
      J_cand <- sample(maxInt, 1, prob = Iw0)
      u_cand <- sample(p, J_cand, replace = FALSE, prob = Zw0)
      s_cand <- 1 - 2 * rbinom(J_cand, 1, 0.5)
      t_cand <- runif(J_cand)

      B_new <- makeBasis(s_cand, u_cand, t_cand, tX, 1)

      if (sum(B_new > 0) >= npart) {
        B_cand <- cbind(B, B_new)

        U_cand <- tryCatch(
          solve(symchol(crossprod(B_cand, Vinv) %*% B_cand + scale / tau * Diagonal(M + 2L))),
          error = function(e) FALSE
        )

        if (isFALSE(U_cand)) {
          log_accept_prob_B <- -Inf
        } else {
          U2_cand <- crossprod(z / v, B_cand) %*% U_cand
          log_accept_prob_B <-
            sum(log(abs(diag(U_cand)))) - sum(log(abs(diag(U)))) +
            1 / (2 * w * scale) * (TCP(U2_cand) - TCP(U2))

          log_accept_prob_B <-
            log_accept_prob_B +
            log(scale) / 2 - log(tau) / 2 + log(lam) +
            log(moveProbs[2]) - log(moveProbs[1]) -
            log(M + 1) - log(maxInt) - lchoose(p, J_cand) -
            log(Iw0[J_cand] / sum(Iw0)) -
            log(dmwnchBass(Zw0, u_cand))
        }
      } else {
        log_accept_prob_B <- -Inf
      }

      if (is.nan(as.numeric(log_accept_prob_B))) {
        browser()
      }

      cnt1[1] <- cnt1[1] + 1
      if (log(runif(1)) < as.numeric(log_accept_prob_B)) {
        cnt2[1] <- cnt2[1] + 1
        B <- B_cand
        U <- U_cand
        U2 <- U2_cand
        Iw0[J_cand] <- Iw0[J_cand] + 1
        Zw0[u_cand] <- Zw0[u_cand] + 1
        M <- M + 1L
        lookup[[nlook]] <- list(J = J_cand, s = s_cand, t = t_cand, u = u_cand)
        basis_index <- c(basis_index, nlook)
        nlook <- nlook + 1L
      }
    }

    # -------------------------------------------------------------------------
    # Death step
    # -------------------------------------------------------------------------
    if (move == "D") {
      m0 <- sample(M, 1)
      basis_cand <- lookup[[basis_index[m0]]]
      J_cand <- basis_cand$J
      u_cand <- basis_cand$u

      Zw_cand <- Zw0
      Zw_cand[u_cand] <- Zw_cand[u_cand] - 1
      B_cand <- B[, -(m0 + 1), drop = FALSE]

      U_cand <- tryCatch(
        solve(symchol(crossprod(B_cand, Vinv) %*% B_cand + scale / tau * Diagonal(M))),
        error = function(e) FALSE
      )

      if (isFALSE(U_cand)) {
        log_accept_prob_D <- -Inf
      } else {
        U2_cand <- crossprod(z / v, B_cand) %*% U_cand
        log_accept_prob_D <-
          sum(log(abs(diag(U_cand)))) - sum(log(abs(diag(U)))) +
          1 / (2 * w * scale) * (TCP(U2_cand) - TCP(U2))

        log_accept_prob_D <-
          log_accept_prob_D -
          log(scale) / 2 + log(tau) / 2 - log(lam) -
          log(moveProbs[2]) + log(moveProbs[1]) +
          log(M) + log(maxInt) + lchoose(p, J_cand) +
          log((Iw0[J_cand] - 1) / (sum(Iw0) - 1)) +
          log(dmwnchBass(Zw_cand, u_cand))
      }

      cnt1[2] <- cnt1[2] + 1
      if (is.nan(as.numeric(log_accept_prob_D))) {
        browser()
      }

      if (log(runif(1)) < as.numeric(log_accept_prob_D)) {
        cnt2[2] <- cnt2[2] + 1
        B <- B_cand
        U <- U_cand
        U2 <- U2_cand
        Iw0[J_cand] <- Iw0[J_cand] - 1
        Zw0 <- Zw_cand
        M <- M - 1L
        basis_index <- basis_index[-m0]
      }
    }

    # -------------------------------------------------------------------------
    # Mutation step
    # -------------------------------------------------------------------------
    if (move == "M") {
      m0 <- sample(M, 1)
      basis_cand <- lookup[[basis_index[m0]]]
      J_cand <- basis_cand$J
      u_cand <- basis_cand$u
      s_cand <- 1 - 2 * rbinom(J_cand, 1, 0.5)
      t_cand <- runif(J_cand)

      B_new <- makeBasis(s_cand, u_cand, t_cand, tX, 1)

      if (sum(B_new > 0) > npart) {
        B_cand <- B
        B_cand[, m0 + 1] <- B_new

        U_cand <- tryCatch(
          solve(symchol(crossprod(B_cand, Vinv) %*% B_cand + scale / tau * Diagonal(M + 1L))),
          error = function(e) FALSE
        )

        if (isFALSE(U_cand)) {
          log_accept_prob_M <- -Inf
        } else {
          U2_cand <- crossprod(z / v, B_cand) %*% U_cand
          log_accept_prob_M <-
            sum(log(abs(diag(U_cand)))) - sum(log(abs(diag(U)))) +
            1 / (2 * w * scale) * (sum(U2_cand^2) - sum(U2^2))
        }
      } else {
        log_accept_prob_M <- -Inf
      }

      if (is.nan(as.numeric(log_accept_prob_M))) {
        browser()
      }

      cnt1[3] <- cnt1[3] + 1
      if (log(runif(1)) < as.numeric(log_accept_prob_M)) {
        cnt2[3] <- cnt2[3] + 1
        B <- B_cand
        U <- U_cand
        U2 <- U2_cand
        lookup[[nlook]] <- list(J = J_cand, s = s_cand, t = t_cand, u = u_cand)
        basis_index[m0] <- nlook
        nlook <- nlook + 1L
      }
    }

    # -------------------------------------------------------------------------
    # Gibbs / Metropolis updates
    # -------------------------------------------------------------------------
    a <- TCP(U, U2) + sqrt(scale * w) * (U %*% rnorm(M + 1L))
    yhat <- B %*% a
    r <- y - yhat
    pen <- sum(a^2)

    lam <- rgamma(1, a_lambda + M, b_lambda + 1)
    tau <- 1 / rgamma(1, a_tau + (M + 1L) / 2, b_tau + pen / (2 * w))

    bet <- rnorm(
      1,
      (s_beta^2 * sum(r) / sqrt(w) + scale * m_beta) / (s_beta^2 * sum(v) + scale),
      sqrt(scale * s_beta^2 / (s_beta^2 * sum(v) + scale))
    )

    gam <- rnorm(
      1,
      (s_gamma^2 * N + m_gamma) / (s_gamma^2 * sum(v) + 1),
      s_gamma / sqrt(s_gamma^2 * sum(v) + 1)
    )

    # Update w
    rss_over_v <- sum(r^2 / v) / scale
    sum_r <- sum(r)
    quad_term <- w_prior$b + rss_over_v + pen / tau
    shape_shift <- (N + M + 1L) / 2

    if (w_prior$type == "GIG") {
      if (abs(bet) < 1e-9) {
        w_cand <- rgig2(
          p = w_prior$p - shape_shift,
          a = w_prior$a,
          b = quad_term
        )

        if (w_cand >= w_prior$lower_bound) {
          w <- w_cand
        }
      } else {
        w_tform <- rMHN(
          n = 1,
          alpha = N + M - 2 * w_prior$p + 1,
          beta = 0.5 * quad_term,
          gamma = bet * sum_r / scale
        )

        w <- w_tform^(-2)
      }
    } else {
      w_cand <- exp(log(w) + rnorm(1, 0, w_prior$prop_sigma))

      log_alpha_w <-
        (w_prior$p * w_prior$a - shape_shift) * (log(w_cand) - log(w)) -
        0.5 * (rss_over_v + pen / tau) * (1 / w_cand - 1 / w) -
        (w_prior$a + w_prior$b) *
        (log(1 + w_cand^w_prior$p) - log(1 + w^w_prior$p)) +
        (bet / scale) * sum_r * (1 / sqrt(w_cand) - 1 / sqrt(w)) +
        log(w_cand > w_prior$lower_bound)

      if (log_alpha_w > log(runif(1))) {
        cntw <- cntw + 1
        w <- w_cand
      }

      if (w_prior$adapt) {
        if (adapt_iter[k]) {
          theta <- log(w)

          n_adapt <- w_prior$count + 1L
          S1_adapt <- w_prior$sum1 + theta

          w_prior$count <- n_adapt
          w_prior$sum1 <- S1_adapt

          xbar_n <- S1_adapt / n_adapt
          xbar_n_1 <- (S1_adapt - theta) / (n_adapt - 1L)
          w_prior$welford <- w_prior$welford + (theta - xbar_n_1) * (theta - xbar_n)

          if (k > w_prior$adapt_delay) {
            var_adapt <- max(w_prior$welford / (n_adapt - 1L), 0)
            ps_adapt <- sqrt(2.38^2 * var_adapt + 1e-8)
            w_prior$prop_sigma <- min(max(ps_adapt, 1e-3), 5)
          }
        }
      }
    }

    # Update latent local scales v
    v <- rgig2.vec(
      p = v_prior$p - 1 / 2,
      a = v_prior$a + bet^2 / scale,
      b = as.numeric(v_prior$b + r^2 / (w * scale))
    )

    z <- y - bet * v * sqrt(w)
    Vinv <- Matrix::Diagonal(x = 1 / v)
    U <- solve(symchol(crossprod(B, Vinv) %*% B + scale / tau * Diagonal(M + 1L)))
    U2 <- crossprod(z / v, B) %*% U

    v_prior$a <- gam^2
    mu_v <- mu_gig(v_prior$p, v_prior$a, v_prior$b)
    s2_v <- var_gig(v_prior$p, v_prior$a, v_prior$b)

    bias <- sqrt(w) * bet * mu_v
    s2 <- scale * w * mu_v + w * bet^2 * s2_v

    if (keep_iter[k]) {
      ss[kk] <- mean((y - yhat)^2)
      M_mc[kk] <- M
      lam_mc[kk] <- lam
      tau_mc[kk] <- tau
      w_mc[kk] <- w
      bet_mc[kk] <- bet
      gam_mc[kk] <- gam
      v_mc[kk, ] <- v
      basis_mc[[kk]] <- basis_index
      a_mc[[kk]] <- a
      bias_mc[kk] <- bias
      s2_mc[kk] <- s2
      kk <- kk + 1L
    }

    if (verbose && k %% 1000 == 0) {
      pr <- c("MCMC iteration", k, myTimestamp(), "nbasis:", M)
      cat(pr, "\n")
    }
  }

  obj <- list(
    nbasis = M_mc,
    M = M_mc,
    w = w_mc,
    v = v_mc,
    tau = tau_mc,
    lam = lam_mc,
    lamb = lam_mc,
    a = a_mc,
    beta = bet_mc,
    gamma = gam_mc,
    basis = basis_mc,
    lookup = lookup,
    cnt1 = cnt1,
    cnt2 = cnt2,
    cntw = cntw,
    ss = ss,
    v_prior = v_prior,
    X = X,
    y = y,
    scale = scale,
    s2 = s2_mc,
    bias = bias_mc
  )

  class(obj) <- c("nwbass", "gbass")
  obj
}


#' @export
#' @param ... (for backwards compatability)
#' @rdname nwbass
nwbass2 <- function(...) {
  .Deprecated("nwbass")
  nwbass(...)
}

test_that("NW Regression with GBASS", {
  cat('simple NW regression example')

  set.seed(1)
  ff1 <- function(x) 10.391*((x[1]-0.4)*(x[2]-0.6) + 0.36)
  X <- lhs::maximinLHS(300, 2)
  y <- 6*apply(X, 1, ff1) + (rgamma(300, 1.5, 1) - 1.5)
  mod <- nwbass(X, y,
               s_beta=10, m_gamma=100, s_gamma=30,
               nmcmc=2000, nburn=1001)

  X2 <- lhs::maximinLHS(100, 2)
  y2 <- 6*apply(X2, 1, ff1) #+ (rgamma(100, 1.5, 1) - 1.5)
  yhat <- apply(predict(mod, X2)
                ,2, mean)
  d1 <- sqrt(mean((y2-yhat)^2))
  expect_that(d1, is_less_than(sd(y2)/3))
})

#' TBASS - Bayesian MARS with a Student's t likelihood
#'
#' Wrapper around \code{gbass()} for a Student's t error model.
#'
#' @param X An \eqn{N \times p} numeric matrix of predictor variables.
#' @param y A numeric response vector of length \eqn{N}.
#' @param df Degrees of freedom. Default is \code{5}.
#' @param ... Additional arguments passed to \code{gbass()}.
#'
#' @details
#' Uses an inverse-gamma latent-scale representation through a GIG prior on the
#' local variance components. If \code{df > 2}, the scale is set to
#' \code{(df - 2) / df} so that the marginal variance matches \code{w}.
#'
#' @return
#' An object of class \code{c("tbass", "gbass")} containing posterior draws
#' and fitted model information.
#'
#' @export
tbass <- function(X, y, df = 5, ...) {
  v_prior <- list(type = "GIG", p = -df / 2, a = 0, b = df)
  scale <- ifelse(df > 2, (df - 2) / df, 1)
  md <- gbass(X, y, v_prior = v_prior, scale = scale, ...)
  md$df <- df
  class(md) <- c("tbass", "gbass")
  md
}


#' HBASS - Bayesian MARS with Horseshoe or Strawderman-Berger likelihood
#'
#' Wrapper around \code{gbass()} using generalized beta prime latent-scale priors.
#'
#' @param X An \eqn{N \times p} numeric matrix of predictor variables.
#' @param y A numeric response vector of length \eqn{N}.
#' @param w_prior A named list specifying the prior for the global variance
#'   component. See Details.
#' @param likelihood Character string specifying the likelihood family.
#'   Use \code{"h"} for Horseshoe or \code{"sb"} for Strawderman-Berger.
#' @param ... Additional arguments passed to \code{gbass()}.
#'
#' @details
#' The Horseshoe and Strawderman-Berger likelihoods can be expressed using
#' generalized beta prime latent-scale distributions. This function provides a
#' convenient wrapper for these likelihoods. For additional flexibility, use
#' \code{gbass()} directly.
#'
#' @return
#' An object of class \code{c("hbass", "gbass")} containing posterior draws
#' and fitted model information.
#'
#' @export
hbass <- function(X, y,
                  w_prior = list(type = "GBP", p = 1, a = 1 / 2, b = 1 / 2),
                  likelihood = "h", ...) {

  if (likelihood == "h" || likelihood == "horseshoe") {
    v_prior <- list(type = "GBP", p = 1, a = 1 / 2, b = 1 / 2)
  } else if (likelihood == "sb" || likelihood == "strawderman-berger") {
    v_prior <- list(type = "GBP", p = 1 / 2, a = 1, b = 1 / 2)
  } else {
    stop("likelihood not recognized. See documentation.")
  }

  scale <- var(y)
  md <- gbass(X, y, w_prior = w_prior, v_prior = v_prior, scale = scale, ...)
  class(md) <- c("hbass", "gbass")
  md
}

bass_shoe <- hbass


#' QBASS - Bayesian MARS with an asymmetric Laplace likelihood
#'
#' Wrapper around \code{gbass()} for quantile regression.
#'
#' @param X An \eqn{N \times p} numeric matrix of predictor variables.
#' @param y A numeric response vector of length \eqn{N}.
#' @param q Quantile of interest. Default is \code{0.5} for median regression.
#' @param w_prior Prior for the global variance factor.
#' @param ... Additional arguments passed to \code{gbass()}.
#'
#' @details
#' Performs quantile regression for quantile \code{q} using the asymmetric
#' Laplace representation. For many quantiles, fitting separate models in
#' parallel may be convenient.
#'
#' @return
#' An object of class \code{c("qbass", "gbass")} containing posterior draws
#' and fitted model information.
#'
#' @export
qbass <- function(X, y, q = 0.5,
                  w_prior = list(type = "GIG", p = -0.1, a = 0, b = 0.1), ...) {

  dots <- list(...)

  if ("prop_sigma_v" %in% names(dots)) {
    warning("prop_sigma_v is deprecated and ignored.")
    dots$prop_sigma_v <- NULL
  }

  v_prior <- list(type = "GIG", p = 1, a = 2, b = 0)
  scale <- 2 / (q * (1 - q))
  m_beta <- 1 / q - 1 / (1 - q)
  s_beta <- 0

  md <- do.call(
    gbass,
    c(
      list(
        X = X,
        y = y,
        v_prior = v_prior,
        w_prior = w_prior,
        scale = scale,
        m_beta = m_beta,
        s_beta = s_beta
      ),
      dots
    )
  )

  md$q <- q
  class(md) <- c("qbass", "gbass")
  md
}

#' Random generation from the modified half-normal distribution
#'
#' Generates random samples from the modified half-normal distribution with
#' density proportional to
#' \eqn{x^{\alpha - 1} \exp(-\beta x^2 + \gamma x)}
#' for \eqn{x > 0}.
#'
#' @param n Number of random draws.
#' @param alpha Positive shape parameter.
#' @param beta Positive rate parameter multiplying x^2.
#' @param gamma Real-valued linear parameter multiplying x.
#'
#' @return A numeric vector of length \code{n} containing random draws from the
#'   modified half-normal distribution.
#'
#' @details
#' This distribution arises in latent-scale updates for the generalized
#' Bayesian adaptive smoothing spline model. The function is intended mainly
#' for internal model fitting, but it may also be useful on its own.
#'
#' @examples
#' x <- rmhn(1000, alpha = 3, beta = 1, gamma = 0.5)
#' hist(x, breaks = 30, freq = FALSE)
#'
#' @export
rmhn <- function(n = 1, alpha, beta, gamma) {
  .mhn_validate_args(n = n, alpha = alpha, beta = beta, gamma = gamma)

  if (n == 0L) {
    return(numeric(0))
  }

  if (gamma <= 0) {
    return(.mhn_sample_negative_gamma(
      n = n, alpha = alpha, beta = beta, gamma = gamma
    ))
  }

  if (alpha > 1) {
    log_k1 <- .mhn_log_k1(alpha = alpha, beta = beta, gamma = gamma)
    log_k2 <- .mhn_log_k2(alpha = alpha, beta = beta, gamma = gamma)

    if (log_k2 <= log_k1) {
      return(.mhn_sample_positive_gamma_sqrt_gamma(
        n = n, alpha = alpha, beta = beta, gamma = gamma
      ))
    }

    return(.mhn_sample_positive_gamma_normal(
      n = n, alpha = alpha, beta = beta, gamma = gamma
    ))
  }

  .mhn_sample_positive_gamma_general(
    n = n, alpha = alpha, beta = beta, gamma = gamma
  )
}

# For internal use
rMHN <- rmhn

.mhn_validate_args <- function(n, alpha, beta, gamma) {
  if (!is.numeric(n) || length(n) != 1L || is.na(n) || n < 0 || n != as.integer(n)) {
    stop("n must be a nonnegative integer.")
  }
  if (!is.numeric(alpha) || length(alpha) != 1L || is.na(alpha) || alpha <= 0) {
    stop("alpha must be a positive scalar.")
  }
  if (!is.numeric(beta) || length(beta) != 1L || is.na(beta) || beta <= 0) {
    stop("beta must be a positive scalar.")
  }
  if (!is.numeric(gamma) || length(gamma) != 1L || is.na(gamma)) {
    stop("gamma must be a finite scalar.")
  }
  invisible(NULL)
}

.mhn_rejection_fill <- function(n, draw_fn, accept_fn) {
  out <- numeric(0)

  while (length(out) < n) {
    need <- n - length(out)
    proposal <- draw_fn(need)
    keep <- accept_fn(proposal)
    out <- c(out, proposal[keep])
  }

  out
}

.mhn_sample_truncated_normal_positive <- function(n, mean, sd) {
  out <- numeric(0)

  while (length(out) < n) {
    need <- n - length(out)
    x <- stats::rnorm(need, mean = mean, sd = sd)
    out <- c(out, x[x > 0])
  }

  out
}

.mhn_log_k1 <- function(alpha, beta, gamma) {
  mu <- (gamma + sqrt(gamma^2 + 8 * beta * (alpha - 1))) / (4 * beta)

  log(2 * sqrt(pi)) +
    (alpha - 1) * (log(sqrt(beta) * (alpha - 1)) - log(2 * beta * mu - gamma)) -
    (alpha - 1) +
    beta * mu^2
}

.mhn_log_k2 <- function(alpha, beta, gamma) {
  delta <- beta + (gamma^2 - gamma * sqrt(gamma^2 + 8 * alpha * beta)) / (4 * alpha)

  (alpha / 2) * log(beta) +
    lgamma(alpha / 2) +
    gamma^2 / (4 * (beta - delta)) -
    (alpha / 2) * log(delta)
}

.mhn_sample_positive_gamma_sqrt_gamma <- function(n, alpha, beta, gamma) {
  delta <- beta + (gamma^2 - sqrt(gamma^4 + 8 * alpha * beta * gamma^2)) / (4 * alpha)

  .mhn_rejection_fill(
    n = n,
    draw_fn = function(m) sqrt(stats::rgamma(m, shape = alpha / 2, rate = delta)),
    accept_fn = function(x) {
      log(stats::runif(length(x))) <=
        (-(beta - delta) * x^2 + gamma * x - gamma^2 / (4 * (beta - delta)))
    }
  )
}

.mhn_sample_positive_gamma_normal <- function(n, alpha, beta, gamma) {
  mode <- (gamma + sqrt(gamma^2 + 8 * beta * (alpha - 1))) / (4 * beta)
  sd <- 1 / sqrt(2 * beta)

  .mhn_rejection_fill(
    n = n,
    draw_fn = function(m) .mhn_sample_truncated_normal_positive(m, mean = mode, sd = sd),
    accept_fn = function(x) {
      log(stats::runif(length(x))) <=
        ((alpha - 1) * log(x / mode) + (2 * beta * mode - gamma) * (mode - x))
    }
  )
}

.mhn_sample_positive_gamma_general <- function(n, alpha, beta, gamma,
                                               tail_tol = 1e-12,
                                               ratio_cap = 0.5,
                                               max_terms = 10000L) {
  weights <- .mhn_positive_gamma_weights(
    alpha = alpha,
    beta = beta,
    gamma = gamma,
    tail_tol = tail_tol,
    ratio_cap = ratio_cap,
    max_terms = max_terms
  )

  w <- sample.int(length(weights), size = n, replace = TRUE, prob = weights) - 1L
  shape <- (alpha + w) / 2
  sqrt(stats::rgamma(n, shape = shape, rate = beta))
}

.mhn_positive_gamma_weights <- function(alpha, beta, gamma,
                                        tail_tol = 1e-12,
                                        ratio_cap = 0.5,
                                        max_terms = 10000L) {
  log_q <- numeric(max_terms)
  log_q[1] <- lgamma(alpha / 2)

  k <- 0L
  ratio <- Inf
  tail_bound <- Inf
  log_q_max <- log_q[1]

  repeat {
    k <- k + 1L
    if (k >= max_terms) {
      stop("Maximum number of mixture terms reached in .mhn_positive_gamma_weights().")
    }

    log_ratio <-
      log(gamma / sqrt(beta)) +
      lgamma((alpha + k) / 2) -
      lgamma((alpha + k - 1) / 2) -
      log(k)

    log_q[k + 1L] <- log_q[k] + log_ratio
    log_q_max <- max(log_q_max, log_q[k + 1L])

    ratio <- exp(log_ratio)

    if (ratio <= ratio_cap) {
      tail_bound <- exp(log_q[k + 1L] - log_q_max) * ratio / (1 - ratio)

      partial_sum <- sum(exp(log_q[seq_len(k + 1L)] - log_q_max))

      if (tail_bound <= tail_tol * partial_sum) {
        break
      }
    }
  }

  q <- exp(log_q[seq_len(k + 1L)] - log_q_max)
  q / sum(q)
}

.mhn_sample_negative_gamma <- function(n, alpha, beta, gamma) {
  if (gamma > 0) {
    stop("Internal error: gamma must satisfy gamma <= 0.")
  }

  a <- beta
  b <- abs(gamma)

  if (alpha <= 1) {
    shape_par <- ((a + b) / (2 * a + b)) * alpha
    rate_par <- a + b
    exponent_power <- (a + b) / (2 * a + b)

    return(.mhn_rejection_fill(
      n = n,
      draw_fn = function(m) stats::rgamma(m, shape = shape_par, rate = rate_par)^exponent_power,
      accept_fn = function(x) {
        x_base <- x^(1 / exponent_power)
        log(stats::runif(length(x))) <=
          (-a * x^2 - b * x + (a + b) * x_base)
      }
    ))
  }

  x_mode <- (sqrt(b^2 + 8 * (alpha - 1) * a) - b) / (4 * a)
  scale <- x_mode

  new_a <- a * scale^2
  new_b <- b * scale

  shape_par <- ((new_a + new_b) / (2 * new_a + new_b)) * alpha
  rate_par <- new_a + new_b
  exponent_power <- (new_a + new_b) / (2 * new_a + new_b)

  x_scaled <- .mhn_rejection_fill(
    n = n,
    draw_fn = function(m) stats::rgamma(m, shape = shape_par, rate = rate_par)^exponent_power,
    accept_fn = function(x) {
      x_base <- x^(1 / exponent_power)
      log(stats::runif(length(x))) <=
        (-new_a * x^2 - new_b * x + (new_a + new_b) * x_base)
    }
  )

  x_scaled * scale
}

#' Construct a prior specification for GBASS
#'
#' @param type "GIG" or "GBP"
#' @param p,a,b prior hyperparameters
#' @param lower_bound optional lower bound
#' @param prop_sigma optional proposal sd (log scale)
#' @param adapt logical; use adaptive MH?
#' @param adapt_delay delay before adapting
#' @param adapt_thin thinning for adaptation updates
#'
#' @return A named list containing the prior specification.
#'
#' @export
build_prior <- function(type = c("GIG", "GBP"),
                        p,
                        a,
                        b,
                        lower_bound = NULL,
                        prop_sigma = NULL,
                        adapt = NULL,
                        adapt_delay = NULL,
                        adapt_thin = NULL) {

  type <- match.arg(type)

  if (!is.numeric(p) || length(p) != 1L || is.na(p)) {
    stop("p must be a numeric scalar.")
  }
  if (!is.numeric(a) || length(a) != 1L || is.na(a)) {
    stop("a must be a numeric scalar.")
  }
  if (!is.numeric(b) || length(b) != 1L || is.na(b)) {
    stop("b must be a numeric scalar.")
  }

  out <- list(
    type = type,
    p = p,
    a = a,
    b = b
  )

  if (!is.null(lower_bound)) {
    if (!is.numeric(lower_bound) || length(lower_bound) != 1L || is.na(lower_bound)) {
      stop("lower_bound must be a numeric scalar.")
    }
    out$lower_bound <- lower_bound
  }

  if (!is.null(prop_sigma)) {
    if (!is.numeric(prop_sigma) || length(prop_sigma) != 1L ||
        is.na(prop_sigma) || prop_sigma <= 0) {
      stop("prop_sigma must be a positive numeric scalar.")
    }
    out$prop_sigma <- prop_sigma
  }

  if (!is.null(adapt)) {
    if (!is.logical(adapt) || length(adapt) != 1L || is.na(adapt)) {
      stop("adapt must be a logical scalar.")
    }
    out$adapt <- adapt
  }

  if (!is.null(adapt_delay)) {
    if (!is.numeric(adapt_delay) || length(adapt_delay) != 1L ||
        is.na(adapt_delay) || adapt_delay < 0 ||
        adapt_delay != as.integer(adapt_delay)) {
      stop("adapt_delay must be a nonnegative integer.")
    }
    out$adapt_delay <- as.integer(adapt_delay)
  }

  if (!is.null(adapt_thin)) {
    if (!is.numeric(adapt_thin) || length(adapt_thin) != 1L ||
        is.na(adapt_thin) || adapt_thin < 1 ||
        adapt_thin != as.integer(adapt_thin)) {
      stop("adapt_thin must be a positive integer.")
    }
    out$adapt_thin <- as.integer(adapt_thin)
  }

  out
}

#' @rdname build_prior
#' @export
build_GIG <- function(p, a, b,
                      lower_bound = NULL,
                      prop_sigma = NULL,
                      adapt = NULL,
                      adapt_delay = NULL,
                      adapt_thin = NULL) {

  build_prior(
    type = "GIG",
    p = p,
    a = a,
    b = b,
    lower_bound = lower_bound,
    prop_sigma = prop_sigma,
    adapt = adapt,
    adapt_delay = adapt_delay,
    adapt_thin = adapt_thin
  )
}

#' @rdname build_prior
#' @export
build_GBP <- function(p, a, b,
                      lower_bound = NULL,
                      prop_sigma = NULL,
                      adapt = NULL,
                      adapt_delay = NULL,
                      adapt_thin = NULL) {

  build_prior(
    type = "GBP",
    p = p,
    a = a,
    b = b,
    lower_bound = lower_bound,
    prop_sigma = prop_sigma,
    adapt = adapt,
    adapt_delay = adapt_delay,
    adapt_thin = adapt_thin
  )
}

#' Calibrate a prior for the Normal-Wald gamma parameter
#'
#' Selects prior hyperparameters for \code{gamma} in the Normal-Wald model using
#' probability statements about the steepness parameter
#' \eqn{\xi = (1 + \gamma)^{-1/2}}.
#'
#' @param q1 Lower reference value for the steepness parameter. Default is
#'   \code{0.1}.
#' @param q2 Upper reference value for the steepness parameter. Default is
#'   \code{0.9}.
#' @param p1 Target probability associated with \code{q1}. Default is
#'   \code{0.5}.
#' @param p2 Target probability associated with \code{q2}. Default is
#'   \code{0.05}.
#' @param par0 Optional starting values for the numerical optimization.
#' @param lambda Optional ridge penalty used in the optimization. Default is
#'   \code{0}.
#'
#' @return A named numeric vector with entries \code{m_gamma} and
#'   \code{s_gamma}.
#'
#' @details
#' This function chooses hyperparameters for a prior on \code{gamma} by solving
#' a simple calibration problem based on the steepness parameter
#' \eqn{(1 + \gamma)^{-1/2}}. The optimization is carried out with
#' \code{\link[stats]{optim}} using the Nelder-Mead method.
#'
#' @export
#'
#' @examples
#' nw_gamma_prior()
#' nw_gamma_prior(q1 = 0.2, q2 = 0.8, p1 = 0.4, p2 = 0.1)
nw_gamma_prior <- function(q1=0.1, q2=0.9, p1=0.5, p2=0.05, par0=NULL, lambda=0){
  logit <- function(z) log(z/(1-z))
  fff <- function(xx, q1,q2,p1,p2,lambda){
    lhs1 <- 1 + pnorm((1-q1^-2-xx[1])/xx[2]) - pnorm((q1^-2-1-xx[1])/xx[2])
    lhs2 <- pnorm((q2^-2-1-xx[1])/xx[2]) - pnorm((1-q2^-2-1-xx[1])/xx[2])
    (logit(lhs1)-logit(p1))^2 + (logit(lhs2)-logit(p2))^2 + lambda*sum(xx^2)
  }
  if(is.null(par0)) par0 <- c(100, 100)
  opt <- optim(par0, fff, method="Nelder-Mead", q1=q1, q2=q2, p1=p1, p2=p2, lambda=lambda, control=list(maxit=c(50000)))
  par <- opt$par
  names(par) <- c("m_gamma", "s_gamma")
  return(par)
}

#' Method-of-moments estimation for the Normal-Wald distribution
#'
#' Estimates Normal-Wald parameters from sample moments or from a supplied
#' vector of moments.
#'
#' @param data Optional data vector. If supplied, empirical moments are computed
#'   from \code{data} and \code{stats} is ignored.
#' @param stats Optional numeric vector of length 4 containing mean, variance,
#'   skewness, and kurtosis. Entries may be \code{NA} when the corresponding
#'   parameter is fixed.
#' @param mu Location parameter, if fixed.
#' @param delta Scale parameter, if fixed.
#' @param beta Skewness parameter, if fixed.
#' @param alpha Tail parameter, if fixed.
#' @param triangle Logical; if \code{TRUE}, return only the asymmetry and
#'   steepness summary parameters.
#' @param ... Additional arguments passed to \code{\link[stats]{optim}}.
#'
#' @return
#' If \code{triangle = FALSE}, a named numeric vector with entries
#' \code{mu}, \code{delta}, \code{beta}, and \code{alpha}.
#' If \code{triangle = TRUE}, a named numeric vector with entries
#' \code{asymmetry} and \code{steepness}.
#'
#' @details
#' This function computes method-of-moments estimates for the Normal-Wald
#' distribution. If \code{data} is supplied, sample mean, variance, skewness,
#' and kurtosis are computed automatically. Otherwise, the user may provide
#' these moments directly through \code{stats}.
#'
#' If some parameters are fixed, only the remaining parameters are estimated.
#' When \code{triangle = TRUE}, the returned values are the transformed summary
#' quantities
#' \eqn{\text{steepness} = (1 + |\gamma|)^{-1/2}} and a corresponding
#' asymmetry measure.
#'
#' @export
#'
#' @examples
#' n <- 500
#' y <- rgamma(n, 3, 1.5) + rlnorm(n, 1, 0.5)
#' z <- (y - mean(y)) / sd(y)
#' skew <- mean(z^3)
#' kurt <- mean(z^4)
#'
#' nw_est_mom(stats = c(NA, NA, skew, kurt), mu = 0, delta = 1, triangle = TRUE)
#' nw_est_mom(stats = c(NA, var(y), skew, kurt), mu = 0, triangle = TRUE)
nw_est_mom <- function(data=NULL, stats=NULL,
                       mu=NA, delta=NA, beta=NA, alpha=NA,
                       triangle=FALSE, ...){
  if(!is.null(data)){
    zdata <- (data-mean(data))/sd(data)
    stats <- c(mean(data), var(data), mean(zdata^3), mean(zdata^4))
  }
  moments_requested <- which(c(is.na(mu), is.na(delta), is.na(beta), is.na(alpha)))
  moments_provided  <- which(!is.na(stats))

  if(length(moments_requested) > length(moments_provided)){
    stop("Need more moments to estimate desired parameters")
  }
  if(moments_requested[1] == 1 & moments_provided[1] != 1){
    stop("Cannot estimate mu without sample mean")
  }

  f2opt <- function(par, moments, mu=NA, delta=NA, beta=NA, alpha=NA){
    cnt <- 1
    if(is.na(mu)){
      mu <- par[cnt]
      cnt <- cnt + 1
    }
    if(is.na(delta)){
      delta <- exp(par[cnt])
      cnt <- cnt + 1
    }
    if(is.na(beta)){
      beta <- par[cnt]
      cnt <- cnt + 1
    }
    if(is.na(alpha)){
      alpha <- par[cnt]
    }
    if(alpha^2 < beta^2) return(1e7)
    gamma <- sqrt(alpha^2 - beta^2)

    m1 <- mu + delta*beta/gamma
    m2 <- delta*alpha^2/gamma^3
    m3 <- 3*beta/(alpha*sqrt(delta*gamma))
    m4 <- 3 + 3*(1 + 4*beta^2)/(alpha^2*delta*gamma)

    d1 <- d2 <- d3 <- d4 <- 0
    if(!is.na(moments[1])) d1 <- (m1 - moments[1])^2
    if(!is.na(moments[2])) d2 <- (m2 - moments[2])^2
    if(!is.na(moments[3])) d3 <- (m3 - moments[3])^2
    if(!is.na(moments[4])) d4 <- (m4 - moments[4])^2

    d1 + d2 + d3 + d4
  }

  par0 <- c(0, 0, 0, 1)[moments_requested]
  fit <- optim(par0, fn=f2opt, moments= stats, mu=mu, delta=delta, beta=beta, alpha=alpha, ...)
  if(fit$convergence != 0) warning("Optim may not have converged.")

  cnt <- 1
  if(1 %in% moments_requested){
    mu <- fit$par[cnt]
    cnt <- cnt + 1
  }
  if(2 %in% moments_requested){
    delta <- exp(fit$par[cnt])
    cnt <- cnt + 1
  }
  if(3 %in% moments_requested){
    beta <- fit$par[cnt]
    cnt <- cnt + 1
  }
  if(4 %in% moments_requested){
    alpha <- abs(fit$par[cnt])
    cnt <- cnt + 1
  }

  if(triangle){
    gamma <- sqrt(alpha^2 - beta^2)
    steep <- 1 / sqrt(1 + abs(gamma))
    asymm <- beta * steep / alpha
    res <- c(asymm, steep)
    names(res) <- c("asymmetry", "steepness")
  } else {
    res <- c(mu, delta, beta, alpha)
    names(res) <- c("mu", "delta", "beta", "alpha")
  }

  return(res)
}


#' Plot Normal-Wald shape parameters on the asymmetry-steepness triangle
#'
#' Visualizes posterior draws of the Normal-Wald shape parameters in terms of
#' asymmetry and steepness. The plotted coordinates are
#' \eqn{\text{steepness} = (1 + \gamma)^{-1/2}} and
#' \eqn{\text{asymmetry} = \beta (\beta^2 + \gamma^2)^{-1/2} \times \text{steepness}}.
#'
#' @param obj A fitted object containing components \code{beta} and \code{gamma},
#'   typically an object returned by \code{nwbass()}.
#' @param add Logical; if \code{FALSE}, initialize a new triangle plot. If
#'   \code{TRUE}, add points to the current plot.
#' @param details Logical; if \code{TRUE} and \code{add = FALSE}, add a few
#'   reference distributions to the plot.
#' @param ... Additional graphical arguments passed to \code{\link{points}}.
#'
#' @return Invisibly returns a data frame with columns \code{asymmetry} and
#'   \code{steepness}.
#'
#' @details
#' This plot provides a simple geometric summary of the shape of the
#' Normal-Wald likelihood. The upper boundary corresponds to symmetric models,
#' while points away from zero on the horizontal axis indicate asymmetry.
#'
#' @export
#'
#' @examples
#' # mod is a fitted nwbass model
#' \donttest{
#' # Simple example
#' n <- 200
#' X <- lhs::maximinLHS(n, 2)
#' f <- 20 * apply(X, 1, function(x) sin(pi * x[1]) + x[2]^2)
#' eps <- rgamma(n, 3, 1.5) - 2
#' y <- f + eps
#'
#' mod <- nwbass(X, y,
#'               m_beta=0, s_beta=10,
#'               m_gamma = nw_gamma_prior()[1], s_gamma = nw_gamma_prior()[2])
#' nw_triangle(mod)
#' }
nw_triangle <- function(obj, add = FALSE, details = FALSE, ...) {
  if (is.null(obj$beta) || is.null(obj$gamma)) {
    stop("obj must contain components 'beta' and 'gamma'.")
  }

  steepness <- (obj$gamma + 1)^(-1 / 2)
  asymmetry <- obj$beta / sqrt(obj$beta^2 + obj$gamma^2) * steepness

  if (!add) {
    plot(
      NULL,
      xlim = c(-1, 1),
      ylim = c(0, 1),
      xlab = "Asymmetry",
      ylab = "Steepness"
    )

    segments(
      x0 = c(-1, -1, 0),
      x1 = c(1, 0, 1),
      y0 = c(1, 1, 0),
      y1 = c(1, 0, 1),
      lwd = 3,
      col = adjustcolor("gray", alpha.f = 0.5)
    )

    if (details) {
      points(rep(0, 5), c(1, 0.9, 0.5, 0.1428, 0), pch = 0:4, bg = "black")

      legend(
        "bottomright",
        c("Cauchy", "t(5)", "t(10)", "t(100)", "Gaussian",
          "Lnorm(0.5)", "Lnorm(0.1)", "Lnorm(0.01)"),
        pch = c(0:4, 15:17),
        col = 1,
        bty = "n",
        cex = 1.2
      )

      cnt <- 0
      for (a in c(0.5, 0.1, 0.01)) {
        k <- exp(4 * a) + 2 * exp(3 * a) + 3 * exp(2 * a) - 6
        s <- (exp(a) + 2) * sqrt(exp(a) - 1)
        g <- 9 / (3 * k - 4 * s^2)
        p <- s * sqrt(g) / 3

        xi <- (1 + abs(g))^(-1 / 2)
        chi <- p * xi

        points(chi, xi, pch = 15 + cnt)
        cnt <- cnt + 1
      }
    }
  }

  points(asymmetry, steepness, ...)

  invisible(data.frame(asymmetry = asymmetry, steepness = steepness))
}

#' Generalized Bayesian MARS
#'
#' Fits a generalized Bayesian MARS (GBASS) model for nonlinear regression under
#' flexible latent-scale likelihoods.
#'
#' @param X An \eqn{N \times p} numeric matrix of predictors. A numeric vector is
#'   treated as a single-column matrix.
#' @param y A numeric response vector of length \eqn{N}.
#' @param w_prior A named list specifying the prior for the global variance
#'   component. See Details.
#' @param v_prior A named list specifying the shared prior for the local variance
#'   components. See Details.
#' @param maxInt Integer giving the maximum interaction degree in proposed basis
#'   functions.
#' @param maxBasis Maximum number of basis functions.
#' @param npart Minimum number of nonzero points required for a proposed basis
#'   function. Defaults to \code{min(20, 0.1 * N)}.
#' @param nmcmc Total number of MCMC iterations.
#' @param nburn Number of initial iterations discarded as burn-in.
#' @param thin Thinning interval for retained draws.
#' @param moveProbs A length-3 vector giving probabilities for birth, death, and
#'   mutation moves.
#' @param a_tau Prior hyperparameter for \code{tau}.
#' @param b_tau Prior hyperparameter for \code{tau}. Defaults to \code{N / 2}.
#' @param a_lambda Prior hyperparameter for \code{lambda}.
#' @param b_lambda Prior hyperparameter for \code{lambda}.
#' @param m_beta Prior mean for \code{beta}.
#' @param s_beta Prior standard deviation for \code{beta}.
#' @param scale Fixed variance-scale parameter. Defaults to \code{1}.
#' @param Iw0 Vector of positive nominal weights for interaction order in proposed
#'   basis functions. Must have length \code{maxInt}.
#' @param Zw0 Vector of positive nominal weights for variable selection in
#'   proposed basis functions. Must have length \code{ncol(X)}.
#' @param verbose Logical; should progress be printed?
#'
#' @details
#' The priors for \code{w} and \code{v_i} must belong to the generalized inverse
#' Gaussian (\code{"GIG"}) or generalized beta prime (\code{"GBP"}) families.
#'
#' Each prior list may contain:
#' \enumerate{
#'   \item \code{type}: either \code{"GIG"} or \code{"GBP"}.
#'   \item \code{p}, \code{a}, \code{b}: prior hyperparameters.
#'   \item \code{lower_bound}: optional lower bound for the parameter support.
#'     For backward compatibility, \code{lb} is also accepted.
#'   \item \code{prop_sigma}: proposal standard deviation on the log scale for
#'     Metropolis updates, when applicable.
#'   \item \code{adapt}, \code{adapt_delay}, \code{adapt_thin}: optional controls
#'     for adaptive Metropolis updates when applicable.
#' }
#'
#' For \code{w}, \code{prop_sigma} and the adaptive Metropolis fields are mainly
#' relevant when \code{w_prior$type = "GBP"}. In the \code{"GIG"} case with
#' nonzero \code{beta}, \code{w} is sampled using the modified half-normal
#' formulation rather than a random-walk Metropolis step.
#'
#' Retained draws are taken at iterations
#' \code{nburn + 1, nburn + 1 + thin, ...}. Thus \code{nburn} is interpreted as
#' the number of initial iterations discarded as burn-in.
#'
#' @return
#' An object of class \code{"gbass"} containing posterior draws and fitted model
#' information.
#'
#' @note
#' Current implementation notes:
#' \enumerate{
#'   \item Basis function parameters are stored as lists.
#'   \item Knot locations use continuous uniform proposals.
#'   \item Basis coefficients use a ridge-type prior.
#' }
#'
#' @import Matrix
#' @export
#'
#' @examples
#' ff1 <- function(x) 10.391*((x[1]-0.4)*(x[2]-0.6) + 0.36)
#'
#' n <- 100
#' p <- 4
#' X <- matrix(runif(n * p), nrow = n)
#' y <- apply(X, 1, ff1)
#' mod <- gbass(X, y, nmcmc = 1000, nburn = 900, thin = 2)
gbass <- function(
    X, y,
    w_prior = list(type = "GIG", p = 0, a = 0, b = 0),
    v_prior = list(type = "GIG", p = -15, a = 0, b = 30),
    maxInt = 3, maxBasis = 1000, npart = NULL,
    nmcmc = 10000, nburn = 9000, thin = 1,
    moveProbs = rep(1 / 3, 3),
    a_tau = 1 / 2, b_tau = NULL,
    a_lambda = 1, b_lambda = 1,
    m_beta = 0, s_beta = 0,
    scale = 1,
    Iw0 = rep(1, maxInt), Zw0 = rep(1, ncol(X)),
    verbose = TRUE
) {
  # ---------------------------------------------------------------------------
  # Input checks and normalization
  # ---------------------------------------------------------------------------
  if (is.null(dim(X))) {
    X <- matrix(X, ncol = 1)
  } else {
    X <- as.matrix(X)
  }

  if (!is.numeric(X)) {
    stop("X must be numeric.")
  }
  if (!is.numeric(y)) {
    stop("y must be numeric.")
  }
  if (anyNA(X) || anyNA(y)) {
    stop("X and y must not contain missing values.")
  }
  if (nrow(X) != length(y)) {
    stop("nrow(X) and length(y) should match.")
  }
  if (!is.numeric(nmcmc) || length(nmcmc) != 1L || nmcmc < 1 || nmcmc != as.integer(nmcmc)) {
    stop("nmcmc must be a positive integer.")
  }
  if (!is.numeric(nburn) || length(nburn) != 1L || nburn < 0 || nburn != as.integer(nburn)) {
    stop("nburn must be a nonnegative integer.")
  }
  if (nburn >= nmcmc) {
    stop("nburn must be strictly less than nmcmc.")
  }
  if (!is.numeric(thin) || length(thin) != 1L || thin < 1 || thin != as.integer(thin)) {
    stop("thin must be a positive integer.")
  }
  if (!is.numeric(maxBasis) || length(maxBasis) != 1L || maxBasis < 1 || maxBasis != as.integer(maxBasis)) {
    stop("maxBasis must be a positive integer.")
  }
  if (!is.numeric(maxInt) || length(maxInt) != 1L || maxInt < 1 || maxInt != as.integer(maxInt)) {
    stop("maxInt must be a positive integer.")
  }
  if (!is.numeric(scale) || length(scale) != 1L || scale <= 0) {
    stop("scale must be a positive numeric scalar.")
  }
  if (!is.numeric(moveProbs) || length(moveProbs) != 3L || any(moveProbs < 0) || sum(moveProbs) <= 0) {
    stop("moveProbs must be a numeric vector of length 3 with nonnegative entries and positive sum.")
  }
  moveProbs <- moveProbs / sum(moveProbs)

  N <- nrow(X)
  p <- ncol(X)

  if (N < 4) {
    warning("Trying to fit a model with less than 4 data points may fail.")
  }

  maxInt <- min(p, maxInt)

  if (length(Iw0) != maxInt || any(Iw0 <= 0)) {
    stop("Iw0 must have length maxInt and positive entries.")
  }
  if (length(Zw0) != p || any(Zw0 <= 0)) {
    stop("Zw0 must have length ncol(X) and positive entries.")
  }

  if (is.null(w_prior$type) || !(w_prior$type %in% c("GIG", "GBP"))) {
    stop("w_prior$type must be either 'GIG' or 'GBP'.")
  }
  if (is.null(v_prior$type) || !(v_prior$type %in% c("GIG", "GBP"))) {
    stop("v_prior$type must be either 'GIG' or 'GBP'.")
  }

  if (!is.null(w_prior$lb) && is.null(w_prior$lower_bound)) {
    w_prior$lower_bound <- w_prior$lb
  }
  if (!is.null(v_prior$lb) && is.null(v_prior$lower_bound)) {
    v_prior$lower_bound <- v_prior$lb
  }

  keep_idx <- seq.int(from = nburn + 1L, to = nmcmc, by = thin)
  nkeep <- length(keep_idx)
  keep_iter <- rep(FALSE, nmcmc)
  keep_iter[keep_idx] <- TRUE

  if (is.null(npart)) {
    npart <- min(20, 0.1 * N)
  }
  if (is.null(b_tau)) {
    b_tau <- N / 2
  }
  if (is.null(w_prior$lower_bound)) {
    w_prior$lower_bound <- var(y) / N
  }
  if (is.null(v_prior$lower_bound)) {
    v_prior$lower_bound <- 0
  }

  if (min(X) < 0 || max(X) > 1) {
    warning("Data out of range (0, 1). Are you sure you want to do this?")
  }

  # ---------------------------------------------------------------------------
  # Allocate storage
  # ---------------------------------------------------------------------------
  a_mc <- list()
  M_mc <- lam_mc <- tau_mc <- w_mc <- bet_mc <- ss <- bias_mc <- s2_mc <- rep(NA_real_, nkeep)
  v_mc <- matrix(NA_real_, nrow = nkeep, ncol = N)
  lookup <- basis_mc <- list()
  basis_index <- integer(0)
  nlook <- 1L
  kk <- 1L

  # ---------------------------------------------------------------------------
  # Initialize parameters
  # ---------------------------------------------------------------------------
  M <- 0L
  tau <- (b_tau + 0.1) / (a_tau + 0.1)
  v <- rep(1, N)
  w <- 2 * var(y) / scale
  lam <- rgamma(1, a_lambda, b_lambda)
  bet <- rnorm(1, m_beta, s_beta)
  a <- mean(y)

  z <- y - bet * v * sqrt(w)
  B <- matrix(1, nrow = N)
  Vinv <- Matrix::Diagonal(x = 1 / v)
  U <- solve(symchol(crossprod(B, Vinv) %*% B + scale / tau * Diagonal(M + 1L)))
  U2 <- crossprod(z / v, B) %*% U

  if (v_prior$type == "GIG") {
    mu_v <- mu_gig(v_prior$p, v_prior$a, v_prior$b)
    s2_v <- var_gig(v_prior$p, v_prior$a, v_prior$b)
    bias <- sqrt(w) * bet * mu_v
    s2 <- scale * w * mu_v + w * bet^2 * s2_v
  }

  cnt1 <- cnt2 <- rep(0, 3)
  cntw <- 0
  cntv <- 0
  adapt_iter_w <- rep(FALSE, nmcmc)
  adapt_iter_v <- rep(FALSE, nmcmc)

  if (w_prior$type == "GBP") {
    if (is.null(w_prior$prop_sigma)) {
      w_prior$prop_sigma <- min(max(var(y) / scale / 2, 1e-3), 5)
    }
    if (is.null(w_prior$adapt)) {
      w_prior$adapt <- TRUE
    }

    if (w_prior$adapt) {
      if (is.null(w_prior$adapt_delay)) {
        w_prior$adapt_delay <- floor(nburn / 10)
      }
      if (is.null(w_prior$adapt_thin)) {
        w_prior$adapt_thin <- 1L
      }

      w_prior$count <- 1L
      w_prior$sum1 <- log(w)
      w_prior$welford <- 0
      adapt_iter_w <- seq_len(nmcmc) %in% seq.int(1L, nburn, by = w_prior$adapt_thin)
    }
  }

  if (v_prior$type == "GBP") {
    if (is.null(v_prior$prop_sigma)) {
      v_prior$prop_sigma <- 0.05 * var(y)
    }
    if (is.null(v_prior$adapt)) {
      v_prior$adapt <- TRUE
    }

    if (v_prior$adapt) {
      if (is.null(v_prior$adapt_delay)) {
        v_prior$adapt_delay <- floor(nburn / 10)
      }
      if (is.null(v_prior$adapt_thin)) {
        v_prior$adapt_thin <- 1L
      }

      v_prior$count <- 1L
      v_prior$sum1 <- mean(log(v))
      v_prior$welford <- 0
      adapt_iter_v <- seq_len(nmcmc) %in% seq.int(1L, nburn, by = v_prior$adapt_thin)
    }
  }

  tX <- t(X)

  if (verbose) {
    pr <- c("MCMC iteration", 0, myTimestamp(), "nbasis:", M)
    cat(pr, "\n")
  }

  # ---------------------------------------------------------------------------
  # MCMC loop
  # ---------------------------------------------------------------------------
  for (k in seq_len(nmcmc)) {
    move <- move_type(M, maxBasis, moveProbs)

    # -------------------------------------------------------------------------
    # Birth step
    # -------------------------------------------------------------------------
    if (move == "B") {
      J_cand <- sample(maxInt, 1, prob = Iw0)
      u_cand <- sample(p, J_cand, replace = FALSE, prob = Zw0)
      s_cand <- 1 - 2 * rbinom(J_cand, 1, 0.5)
      t_cand <- runif(J_cand)

      B_new <- makeBasis(s_cand, u_cand, t_cand, tX, 1)

      if (sum(B_new > 0) >= npart) {
        B_cand <- cbind(B, B_new)

        U_cand <- tryCatch(
          solve(symchol(crossprod(B_cand, Vinv) %*% B_cand + scale / tau * Diagonal(M + 2L))),
          error = function(e) FALSE
        )

        if (isFALSE(U_cand)) {
          log_accept_prob_B <- -Inf
        } else {
          U2_cand <- crossprod(z / v, B_cand) %*% U_cand
          log_accept_prob_B <-
            sum(log(abs(diag(U_cand)))) - sum(log(abs(diag(U)))) +
            1 / (2 * w * scale) * (TCP(U2_cand) - TCP(U2))

          log_accept_prob_B <-
            log_accept_prob_B +
            log(scale) / 2 - log(tau) / 2 + log(lam) +
            log(moveProbs[2]) - log(moveProbs[1]) -
            log(M + 1) - log(maxInt) - lchoose(p, J_cand) -
            log(Iw0[J_cand] / sum(Iw0)) -
            log(dmwnchBass(Zw0, u_cand))
        }
      } else {
        log_accept_prob_B <- -Inf
      }

      if (is.nan(as.numeric(log_accept_prob_B))) {
        browser()
      }

      cnt1[1] <- cnt1[1] + 1
      if (log(runif(1)) < as.numeric(log_accept_prob_B)) {
        cnt2[1] <- cnt2[1] + 1
        B <- B_cand
        U <- U_cand
        U2 <- U2_cand
        Iw0[J_cand] <- Iw0[J_cand] + 1
        Zw0[u_cand] <- Zw0[u_cand] + 1
        M <- M + 1L
        lookup[[nlook]] <- list(J = J_cand, s = s_cand, t = t_cand, u = u_cand)
        basis_index <- c(basis_index, nlook)
        nlook <- nlook + 1L
      }
    }

    # -------------------------------------------------------------------------
    # Death step
    # -------------------------------------------------------------------------
    if (move == "D") {
      m0 <- sample(M, 1)
      basis_cand <- lookup[[basis_index[m0]]]
      J_cand <- basis_cand$J
      u_cand <- basis_cand$u

      Zw_cand <- Zw0
      Zw_cand[u_cand] <- Zw_cand[u_cand] - 1
      B_cand <- B[, -(m0 + 1), drop = FALSE]

      U_cand <- tryCatch(
        solve(symchol(crossprod(B_cand, Vinv) %*% B_cand + scale / tau * Diagonal(M))),
        error = function(e) FALSE
      )

      if (isFALSE(U_cand)) {
        log_accept_prob_D <- -Inf
      } else {
        U2_cand <- crossprod(z / v, B_cand) %*% U_cand
        log_accept_prob_D <-
          sum(log(abs(diag(U_cand)))) - sum(log(abs(diag(U)))) +
          1 / (2 * w * scale) * (TCP(U2_cand) - TCP(U2))

        log_accept_prob_D <-
          log_accept_prob_D -
          log(scale) / 2 + log(tau) / 2 - log(lam) -
          log(moveProbs[2]) + log(moveProbs[1]) +
          log(M) + log(maxInt) + lchoose(p, J_cand) +
          log((Iw0[J_cand] - 1) / (sum(Iw0) - 1)) +
          log(dmwnchBass(Zw_cand, u_cand))
      }

      cnt1[2] <- cnt1[2] + 1
      if (is.nan(as.numeric(log_accept_prob_D))) {
        browser()
      }

      if (log(runif(1)) < as.numeric(log_accept_prob_D)) {
        cnt2[2] <- cnt2[2] + 1
        B <- B_cand
        U <- U_cand
        U2 <- U2_cand
        Iw0[J_cand] <- Iw0[J_cand] - 1
        Zw0 <- Zw_cand
        M <- M - 1L
        basis_index <- basis_index[-m0]
      }
    }

    # -------------------------------------------------------------------------
    # Mutation step
    # -------------------------------------------------------------------------
    if (move == "M") {
      m0 <- sample(M, 1)
      basis_cand <- lookup[[basis_index[m0]]]
      J_cand <- basis_cand$J
      u_cand <- basis_cand$u
      s_cand <- 1 - 2 * rbinom(J_cand, 1, 0.5)
      t_cand <- runif(J_cand)

      B_new <- makeBasis(s_cand, u_cand, t_cand, tX, 1)

      if (sum(B_new > 0) > npart) {
        B_cand <- B
        B_cand[, m0 + 1] <- B_new

        U_cand <- tryCatch(
          solve(symchol(crossprod(B_cand, Vinv) %*% B_cand + scale / tau * Diagonal(M + 1L))),
          error = function(e) FALSE
        )

        if (isFALSE(U_cand)) {
          log_accept_prob_M <- -Inf
        } else {
          U2_cand <- crossprod(z / v, B_cand) %*% U_cand
          log_accept_prob_M <-
            sum(log(abs(diag(U_cand)))) - sum(log(abs(diag(U)))) +
            1 / (2 * w * scale) * (sum(U2_cand^2) - sum(U2^2))
        }
      } else {
        log_accept_prob_M <- -Inf
      }

      if (is.nan(as.numeric(log_accept_prob_M))) {
        browser()
      }

      cnt1[3] <- cnt1[3] + 1
      if (log(runif(1)) < as.numeric(log_accept_prob_M)) {
        cnt2[3] <- cnt2[3] + 1
        B <- B_cand
        U <- U_cand
        U2 <- U2_cand
        lookup[[nlook]] <- list(J = J_cand, s = s_cand, t = t_cand, u = u_cand)
        basis_index[m0] <- nlook
        nlook <- nlook + 1L
      }
    }

    # -------------------------------------------------------------------------
    # Gibbs / Metropolis updates
    # -------------------------------------------------------------------------
    a <- TCP(U, U2) + sqrt(scale * w) * (U %*% rnorm(M + 1L))
    yhat <- B %*% a
    r <- y - yhat
    pen <- sum(a^2)

    lam <- rgamma(1, a_lambda + M, b_lambda + 1)
    tau <- 1 / rgamma(1, a_tau + (M + 1L) / 2, b_tau + pen / (2 * w))

    bet <- rnorm(
      1,
      (s_beta^2 * sum(r) / sqrt(w) + scale * m_beta) / (s_beta^2 * sum(v) + scale),
      sqrt(scale * s_beta^2 / (s_beta^2 * sum(v) + scale))
    )

    # Update w
    rss_over_v <- sum(r^2 / v) / scale
    sum_r <- sum(r)
    quad_term <- w_prior$b + rss_over_v + pen / tau
    shape_shift <- (N + M + 1L) / 2

    if (w_prior$type == "GIG") {
      if (abs(bet) < 1e-9) {
        w_cand <- rgig2(
          p = w_prior$p - shape_shift,
          a = w_prior$a,
          b = quad_term
        )

        if (w_cand >= w_prior$lower_bound) {
          w <- w_cand
        }
      } else {
        w_tform <- rMHN(
          n = 1,
          alpha = N + M - 2 * w_prior$p + 1,
          beta = 0.5 * quad_term,
          gamma = bet * sum_r / scale
        )

        w <- w_tform^(-2)
      }
    } else {
      w_cand <- exp(log(w) + rnorm(1, 0, w_prior$prop_sigma))

      log_alpha_w <-
        (w_prior$p * w_prior$a - shape_shift) * (log(w_cand) - log(w)) -
        0.5 * (rss_over_v + pen / tau) * (1 / w_cand - 1 / w) -
        (w_prior$a + w_prior$b) *
        (log(1 + w_cand^w_prior$p) - log(1 + w^w_prior$p)) +
        (bet / scale) * sum_r * (1 / sqrt(w_cand) - 1 / sqrt(w)) +
        log(w_cand > w_prior$lower_bound)

      if (log_alpha_w > log(runif(1))) {
        cntw <- cntw + 1
        w <- w_cand
      }

      if (w_prior$adapt) {
        if (adapt_iter_w[k]) {
          theta <- log(w)

          n_adapt <- w_prior$count + 1L
          S1_adapt <- w_prior$sum1 + theta

          w_prior$count <- n_adapt
          w_prior$sum1 <- S1_adapt

          xbar_n <- S1_adapt / n_adapt
          xbar_n_1 <- (S1_adapt - theta) / (n_adapt - 1L)
          w_prior$welford <- w_prior$welford + (theta - xbar_n_1) * (theta - xbar_n)

          if (k > w_prior$adapt_delay) {
            var_adapt <- max(w_prior$welford / (n_adapt - 1L), 0)
            ps_adapt <- sqrt(2.38^2 * var_adapt + 1e-8)
            w_prior$prop_sigma <- min(max(ps_adapt, 1e-3), 5)
          }
        }
      }
    }

    # Update v
    if (v_prior$type == "GIG") {
      v <- rgig2.vec(
        p = v_prior$p - 1 / 2,
        a = v_prior$a + bet^2 / scale,
        b = as.numeric(v_prior$b + r^2 / (w * scale))
      )
    } else {
      v_old <- v
      v <- rgbp.vec(as.numeric(v),
                    v_prior$a, v_prior$b, v_prior$p,
                    v_prior$prop_sigma, v_prior$lower_bound,
                    w, scale, as.numeric(r), bet)
      cntv <- cntv + sum(v != v_old)

      if (v_prior$adapt) {
        if (adapt_iter_v[k]) {
          theta <- mean(log(v))

          n_adapt  <- v_prior$count + 1
          S1_adapt <- v_prior$sum1 + theta

          v_prior$count <- n_adapt
          v_prior$sum1  <- S1_adapt

          xbar_n <- S1_adapt / n_adapt
          xbar_n_1 <- (S1_adapt - theta) / (n_adapt - 1)
          v_prior$welford <- v_prior$welford + (theta - xbar_n_1) * (theta - xbar_n)

          if (k > v_prior$adapt_delay) {
            var_adapt <- max(v_prior$welford / (n_adapt - 1), 0)
            ps_adapt <- sqrt(2.38^2 * var_adapt + 1e-8)
            v_prior$prop_sigma <- min(max(ps_adapt, 1e-3), 5)
          }
        }
      }
    }

    z <- y - bet * v * sqrt(w)
    Vinv <- Matrix::Diagonal(x = 1 / v)
    U <- solve(symchol(crossprod(B, Vinv) %*% B + scale / tau * Diagonal(M + 1)))
    U2 <- crossprod(z / v, B) %*% U

    if (v_prior$type == "GIG") {
      bias <- sqrt(w) * bet * mu_v
      s2 <- scale * w * mu_v + w * bet^2 * s2_v
    }

    if (keep_iter[k]) {
      ss[kk] <- mean((y - yhat)^2)
      M_mc[kk] <- M
      lam_mc[kk] <- lam
      tau_mc[kk] <- tau
      w_mc[kk] <- w
      bet_mc[kk] <- bet
      v_mc[kk, ] <- v
      basis_mc[[kk]] <- basis_index
      a_mc[[kk]] <- a

      if (v_prior$type == "GIG") {
        bias_mc[kk] <- bias
        s2_mc[kk] <- s2
      }

      kk <- kk + 1
    }

    if (verbose && k %% 1000 == 0) {
      pr <- c("MCMC iteration", k, myTimestamp(), "nbasis:", M)
      cat(pr, "\n")
    }
  }

  obj <- list(
    nbasis = M_mc,
    M = M_mc,
    w = w_mc,
    v = v_mc,
    tau = tau_mc,
    lam = lam_mc,
    beta = bet_mc,
    a = a_mc,
    basis = basis_mc,
    lookup = lookup,
    cnt1 = cnt1,
    cnt2 = cnt2,
    acc_rate_w = if (w_prior$type == "GBP") cntw / nmcmc else NA,
    acc_rate_v = if (v_prior$type == "GBP") cntv / (nmcmc * N) else NA,
    ss = ss,
    v_prior = v_prior,
    scale = scale,
    s2 = s2_mc,
    bias = bias_mc,
    X = X,
    y = y
  )

  class(obj) <- "gbass"
  obj
}


#' Predict method for GBASS objects
#'
#' Returns posterior draws of either:
#'   (i) the linear predictor / mean surface, or
#'   (ii) the full posterior predictive distribution.
#'
#' @param object an object of class "gbass" (including subclasses like
#'   "tbass", "qbass", "nwbass")
#' @param newdata a matrix of predictor variables. Defaults to training inputs.
#' @param mcmc.use optional vector indicating which posterior draws to use.
#' @param predictive logical. If TRUE, return posterior predictive draws.
#'   If FALSE, return draws of the linear predictor.
#' @param bias_correct logical. Ignored unless predictive = FALSE.
#'   If TRUE, return the posterior mean response rather than just the linear predictor.
#' @param samples Integer giving the number of predictive samples to generate
#'   per retained MCMC draw when \code{predictive = TRUE}. Ignored when
#'   \code{predictive = FALSE}. Default is \code{1}.
#' @param ... Additional graphical arguments (not used)
#'
#' @details
#' If predictive = FALSE and bias_correct = FALSE, this returns draws of B(x)a.
#'
#' If predictive = FALSE and bias_correct = TRUE, this returns draws of
#'   B(x)a + E(error | posterior draw),
#' i.e. the mean response under the fitted GBASS error model.
#'
#' If predictive = TRUE, this returns posterior predictive draws by simulating
#' a fresh latent local variance v_new and Gaussian draw for each posterior sample.
#'
#' For qbass objects, bias_correct = TRUE is usually not what you want, because
#' qbass is typically being used for quantile regression rather than mean regression.
#'
#' Currently, posterior predictive draws are implemented for GIG-based models.
#' If the fitted object uses a GBP prior for v, predictive = TRUE will stop.
#'
#' @return a matrix with rows corresponding to posterior draws and columns
#'   corresponding to rows of newdata.
#'
#' @export
predict.gbass <- function(object, newdata = NULL, mcmc.use = NULL,
                          predictive = TRUE, bias_correct = FALSE,
                          samples = 1, ...) {

  if (is.null(newdata)) {
    newdata <- object$X
  }
  if (is.null(dim(newdata))) {
    newdata <- matrix(newdata, ncol = ncol(object$X))
  }

  if (!is.numeric(samples) || length(samples) != 1L ||
      is.na(samples) || samples < 1 || samples != as.integer(samples)) {
    stop("samples must be a positive integer.")
  }
  samples <- as.integer(samples)

  N <- nrow(newdata)

  if (is.null(mcmc.use)) {
    if (!is.null(object$M)) {
      mcmc.use <- seq_along(object$M)
    } else if (!is.null(object$nbasis)) {
      mcmc.use <- seq_along(object$nbasis)
    } else {
      stop("Could not determine available MCMC draws from object.")
    }
  }

  if (predictive && isTRUE(bias_correct)) {
    warning("bias_correct is ignored when predictive = TRUE.")
  }

  if (inherits(object, "qbass") && !predictive && isTRUE(bias_correct)) {
    warning("bias_correct=TRUE is usually not what you want for qbass/quantile regression.")
  }

  build_basis_gbass <- function(object, draw_index, newdata) {
    tX <- t(newdata)
    basis_curr <- object$lookup[unlist(object$basis[[draw_index]])]
    B_curr <- matrix(1, nrow = nrow(newdata), ncol = length(basis_curr) + 1)

    if (length(basis_curr) > 0) {
      for (m in seq_along(basis_curr)) {
        thetaB <- basis_curr[[m]]
        B_curr[, m + 1] <- makeBasis(thetaB$s, thetaB$u, thetaB$t, tX, 1)
      }
    }
    B_curr
  }

  draw_v_new <- function(object, draw_index, n) {
    if (inherits(object, "nwbass")) {
      if (is.null(object$gamma)) {
        stop("nwbass object does not contain gamma draws.")
      }
      gam <- object$gamma[draw_index]
      return(replicate(n, rgig2(-1/2, gam^2, 1)))
    }

    vp <- object$v_prior

    if (is.null(vp$type)) {
      stop("object$v_prior$type is missing.")
    }

    if (vp$type == "GIG") {
      return(replicate(n, rgig2(vp$p, vp$a, vp$b)))
    }

    if (vp$type == "GBP") {
      if (!is.null(vp$lower_bound) && vp$lower_bound > 0) {
        stop("predictive=TRUE for GBP priors currently requires lower_bound = 0.")
      }

      u <- stats::rbeta(n, shape1 = vp$a, shape2 = vp$b)
      t <- u / (1 - u)
      return(t^(1 / vp$p))
    }

    stop("predictive=TRUE is currently implemented only for GIG- or GBP-based v_prior objects.")
  }

  mean_v_prior <- function(object, draw_index) {
    if (inherits(object, "nwbass")) {
      if (is.null(object$gamma)) {
        stop("nwbass object does not contain gamma draws.")
      }
      gam <- object$gamma[draw_index]
      return(1 / gam)
    }

    vp <- object$v_prior

    if (vp$type == "GIG") {
      return(mu_gig(vp$p, vp$a, vp$b))
    }

    stop("bias_correct=TRUE currently requires a GIG-based v_prior (or nwbass).")
  }

  ndraw <- length(mcmc.use)
  nout <- if (predictive) ndraw * samples else ndraw
  res <- matrix(NA_real_, nrow = nout, ncol = N)

  jj <- 1
  for (i in mcmc.use) {
    B_curr <- build_basis_gbass(object, i, newdata)
    eta <- as.numeric(B_curr %*% object$a[[i]])

    w_i <- object$w[i]
    beta_i <- if (!is.null(object$beta)) object$beta[i] else 0
    scale_i <- if (!is.null(object$scale)) object$scale else 1

    if (!predictive) {
      if (isTRUE(bias_correct)) {
        mu_v_i <- mean_v_prior(object, i)
        res[jj, ] <- eta + sqrt(w_i) * beta_i * mu_v_i
      } else {
        res[jj, ] <- eta
      }
      jj <- jj + 1
    } else {
      for (s in seq_len(samples)) {
        v_new <- draw_v_new(object, i, N)
        z_new <- rnorm(N)
        eps_new <- sqrt(w_i) * (beta_i * v_new + sqrt(scale_i * v_new) * z_new)
        res[jj, ] <- eta + eps_new
        jj <- jj + 1
      }
    }
  }

  return(res)
}

#' Plot method for gbass objects
#'
#' @param x A fitted object of class \code{"gbass"}.
#' @param ... Additional graphical arguments.
#' @method plot gbass
#' @return No return value, called for its side effects.
#' @export
plot.gbass <- function(x, ...){
  if(!("gbass" %in% class(x)))
    stop("x must be an object of class gbass")
  oldpar <- par(no.readonly = TRUE) # code line i
  on.exit(par(oldpar)) # code line i + 1
  par(mfrow=c(2,2))
  plot(x$M, type='l',
       xlab="MCMC iteration (post-burn)",
       ylab="number of basis functions")
  plot(x$w, type='l',
       xlab="MCMC iteration (post-burn)",
       ylab="global error variance (w)")
  yhat <- apply(predict(x), 2, mean)
  plot(x$y, yhat,
       xlab="observed",
       ylab="posterior mean",
       main="Training Fit")
  resid <- x$y - yhat
  hist(resid,
       main="Posterior mean residuals",
       xlab="residuals",
       ylab="Density",
       breaks="FD",
       freq=FALSE)
  curve(dnorm(x, mean(resid), sd(resid)),
        add=TRUE,
        col='red')
  par(mfrow=c(1,1))
}


#' Print a gbass object
#'
#' @param x An object of class \code{"gbass"}.
#' @param ... Unused.
#' @method print gbass
#' @return The input object, invisibly.
#' @export
print.gbass <- function(x, ...) {
  cat("Generalized Bayesian MARS model\n")

  if (!is.null(x$X)) {
    cat("  Observations:", nrow(x$X), "\n")
    cat("  Predictors:", ncol(x$X), "\n")
  }

  if (!is.null(x$M)) {
    cat("  Saved draws:", length(x$M), "\n")
    cat("  Mean basis functions:", round(mean(x$M, na.rm = TRUE), 2), "\n")
  }

  invisible(x)
}

test_that("GBASS with t likelihood", {
  cat('simple test function')

  set.seed(1)
  ff1 <- function(x) 10.391*((x[1]-0.4)*(x[2]-0.6) + 0.36)
  X <- lhs::maximinLHS(300, 2)
  y <- 6*apply(X, 1, ff1)
  mod <- gbass(X, y, nmcmc=2000, nburn=1001)

  X2 <- lhs::maximinLHS(100, 2)
  y2 <- 6*apply(X2, 1, ff1)
  yhat <- apply(predict(mod, X2)
                , 2, mean)
  d1 <- sqrt(mean((y2-yhat)^2))
  expect_that(d1, is_less_than(sd(y)/3))
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nw_tools.R
\name{nw_triangle}
\alias{nw_triangle}
\title{Plot Normal-Wald shape parameters on the asymmetry-steepness triangle}
\usage{
nw_triangle(obj, add = FALSE, details = FALSE, ...)
}
\arguments{
\item{obj}{A fitted object containing components \code{beta} and \code{gamma},
typically an object returned by \code{nwbass()}.}

\item{add}{Logical; if \code{FALSE}, initialize a new triangle plot. If
\code{TRUE}, add points to the current plot.}

\item{details}{Logical; if \code{TRUE} and \code{add = FALSE}, add a few
reference distributions to the plot.}

\item{...}{Additional graphical arguments passed to \code{\link{points}}.}
}
\value{
Invisibly returns a data frame with columns \code{asymmetry} and
\code{steepness}.
}
\description{
Visualizes posterior draws of the Normal-Wald shape parameters in terms of
asymmetry and steepness. The plotted coordinates are
\eqn{\text{steepness} = (1 + \gamma)^{-1/2}} and
\eqn{\text{asymmetry} = \beta (\beta^2 + \gamma^2)^{-1/2} \times \text{steepness}}.
}
\details{
This plot provides a simple geometric summary of the shape of the
Normal-Wald likelihood. The upper boundary corresponds to symmetric models,
while points away from zero on the horizontal axis indicate asymmetry.
}
\examples{
# mod is a fitted nwbass model
\donttest{
# Simple example
n <- 200
X <- lhs::maximinLHS(n, 2)
f <- 20 * apply(X, 1, function(x) sin(pi * x[1]) + x[2]^2)
eps <- rgamma(n, 3, 1.5) - 2
y <- f + eps

mod <- nwbass(X, y,
              m_beta=0, s_beta=10,
              m_gamma = nw_gamma_prior()[1], s_gamma = nw_gamma_prior()[2])
nw_triangle(mod)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mhn.R
\name{rmhn}
\alias{rmhn}
\title{Random generation from the modified half-normal distribution}
\usage{
rmhn(n = 1, alpha, beta, gamma)
}
\arguments{
\item{n}{Number of random draws.}

\item{alpha}{Positive shape parameter.}

\item{beta}{Positive rate parameter multiplying x^2.}

\item{gamma}{Real-valued linear parameter multiplying x.}
}
\value{
A numeric vector of length \code{n} containing random draws from the
modified half-normal distribution.
}
\description{
Generates random samples from the modified half-normal distribution with
density proportional to
\eqn{x^{\alpha - 1} \exp(-\beta x^2 + \gamma x)}
for \eqn{x > 0}.
}
\details{
This distribution arises in latent-scale updates for the generalized
Bayesian adaptive smoothing spline model. The function is intended mainly
for internal model fitting, but it may also be useful on its own.
}
\examples{
x <- rmhn(1000, alpha = 3, beta = 1, gamma = 0.5)
hist(x, breaks = 30, freq = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbass.R
\name{print.gbass}
\alias{print.gbass}
\title{Print a gbass object}
\usage{
\method{print}{gbass}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"gbass"}.}

\item{...}{Unused.}
}
\value{
The input object, invisibly.
}
\description{
Print a gbass object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nwbass.R
\name{nwbass}
\alias{nwbass}
\alias{nwbass2}
\title{Generalized Bayesian MARS with a Normal-Wald likelihood}
\usage{
nwbass(
  X,
  y,
  w_prior = list(type = "GIG", p = -0.1, a = 0, b = 0.1),
  maxInt = 3,
  maxBasis = 1000,
  npart = NULL,
  nmcmc = 10000,
  nburn = 9000,
  thin = 1,
  moveProbs = rep(1/3, 3),
  a_tau = 1/2,
  b_tau = NULL,
  a_lambda = 1,
  b_lambda = 1,
  m_beta = 0,
  s_beta = 0,
  lag_beta = 1001,
  m_gamma = 1,
  s_gamma = 0,
  scale = 1,
  Iw0 = rep(1, maxInt),
  Zw0 = rep(1, ncol(X)),
  verbose = TRUE
)

nwbass2(...)
}
\arguments{
\item{X}{An \eqn{N \times p} numeric matrix of predictor variables. A numeric
vector is treated as a single-column matrix.}

\item{y}{A numeric response vector of length \eqn{N}.}

\item{w_prior}{A named list specifying the prior for the global variance
component. See Details.}

\item{maxInt}{Integer giving the maximum degree of interaction in spline basis
functions. Defaults to \code{3}.}

\item{maxBasis}{Maximum number of basis functions.}

\item{npart}{Minimum number of nonzero points required for a proposed basis
function. Defaults to \code{min(20, 0.1 * N)}.}

\item{nmcmc}{Total number of MCMC iterations.}

\item{nburn}{Number of initial MCMC iterations discarded as burn-in.}

\item{thin}{Thinning interval for retained draws.}

\item{moveProbs}{A length-3 vector giving probabilities for birth, death, and
mutation moves.}

\item{a_tau}{Prior hyperparameter for \code{tau}.}

\item{b_tau}{Prior hyperparameter for \code{tau}. Defaults to \code{N / 2}.}

\item{a_lambda}{Prior hyperparameter for \code{lambda}.}

\item{b_lambda}{Prior hyperparameter for \code{lambda}.}

\item{m_beta}{Prior mean for \code{beta}.}

\item{s_beta}{Prior standard deviation for \code{beta}.}

\item{lag_beta}{Number of initial iterations for which \code{beta} is fixed at
\code{m_beta}. This is often used to stabilize early sampling.}

\item{m_gamma}{Prior mean for \code{gamma}.}

\item{s_gamma}{Prior standard deviation for \code{gamma}.}

\item{scale}{Fixed variance-scale parameter. Defaults to \code{1}.}

\item{Iw0}{Vector of positive nominal weights for interaction order in proposed
basis functions. Must have length \code{maxInt}.}

\item{Zw0}{Vector of positive nominal weights for variable selection in proposed
basis functions. Must have length \code{ncol(X)}.}

\item{verbose}{Logical; should progress be printed?}

\item{...}{(for backwards compatability)}
}
\value{
An object of class \code{c("nwbass", "gbass")} containing posterior draws and
fitted model information.
}
\description{
Fits a generalized BMARS model under a Normal-Wald likelihood. This provides
flexible nonlinear regression with a unimodal, potentially skewed error model.
}
\details{
The latent local-scale prior is fixed internally to the Normal-Wald form
\eqn{v_i \sim \mathrm{GIG}(-1/2, \gamma^2, 1)}. Unlike \code{gbass()},
\code{nwbass()} does not expose a user-specified \code{v_prior}.

The \code{w_prior} list should contain:
\enumerate{
\item \code{type}: either \code{"GIG"} or \code{"GBP"}.
\item \code{p}, \code{a}, \code{b}: hyperparameters for the prior.
\item \code{lower_bound}: optional lower bound for \code{w}. For backward
compatibility, \code{lb} is also accepted.
\item \code{prop_sigma}: proposal standard deviation on the log scale for
Metropolis updates of \code{w}. This is only used when \code{w} is updated
by Metropolis-Hastings, such as the \code{"GBP"} case.
\item \code{adapt}, \code{adapt_delay}, \code{adapt_thin}: optional controls
for adaptive Metropolis updates of \code{w} when applicable.
}

Retained draws are taken at iterations
\code{nburn + 1, nburn + 1 + thin, ...}. Thus \code{nburn} is interpreted as
the number of initial iterations discarded as burn-in.
}
\examples{
\donttest{
n <- 200
X <- lhs::maximinLHS(n, 2)
f <- 20 * apply(X, 1, function(x) sin(pi * x[1]) + x[2]^2)
eps <- rgamma(n, 3, 1.5) - 2
y <- f + eps

mod <- nwbass(X, y,
              m_beta=0, s_beta=10,
              m_gamma = nw_gamma_prior()[1], s_gamma = nw_gamma_prior()[2])
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers.R
\name{hbass}
\alias{hbass}
\title{HBASS - Bayesian MARS with Horseshoe or Strawderman-Berger likelihood}
\usage{
hbass(
  X,
  y,
  w_prior = list(type = "GBP", p = 1, a = 1/2, b = 1/2),
  likelihood = "h",
  ...
)
}
\arguments{
\item{X}{An \eqn{N \times p} numeric matrix of predictor variables.}

\item{y}{A numeric response vector of length \eqn{N}.}

\item{w_prior}{A named list specifying the prior for the global variance
component. See Details.}

\item{likelihood}{Character string specifying the likelihood family.
Use \code{"h"} for Horseshoe or \code{"sb"} for Strawderman-Berger.}

\item{...}{Additional arguments passed to \code{gbass()}.}
}
\value{
An object of class \code{c("hbass", "gbass")} containing posterior draws
and fitted model information.
}
\description{
Wrapper around \code{gbass()} using generalized beta prime latent-scale priors.
}
\details{
The Horseshoe and Strawderman-Berger likelihoods can be expressed using
generalized beta prime latent-scale distributions. This function provides a
convenient wrapper for these likelihoods. For additional flexibility, use
\code{gbass()} directly.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers.R
\name{qbass}
\alias{qbass}
\title{QBASS - Bayesian MARS with an asymmetric Laplace likelihood}
\usage{
qbass(
  X,
  y,
  q = 0.5,
  w_prior = list(type = "GIG", p = -0.1, a = 0, b = 0.1),
  ...
)
}
\arguments{
\item{X}{An \eqn{N \times p} numeric matrix of predictor variables.}

\item{y}{A numeric response vector of length \eqn{N}.}

\item{q}{Quantile of interest. Default is \code{0.5} for median regression.}

\item{w_prior}{Prior for the global variance factor.}

\item{...}{Additional arguments passed to \code{gbass()}.}
}
\value{
An object of class \code{c("qbass", "gbass")} containing posterior draws
and fitted model information.
}
\description{
Wrapper around \code{gbass()} for quantile regression.
}
\details{
Performs quantile regression for quantile \code{q} using the asymmetric
Laplace representation. For many quantiles, fitting separate models in
parallel may be convenient.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nw_tools.R
\name{nw_est_mom}
\alias{nw_est_mom}
\title{Method-of-moments estimation for the Normal-Wald distribution}
\usage{
nw_est_mom(
  data = NULL,
  stats = NULL,
  mu = NA,
  delta = NA,
  beta = NA,
  alpha = NA,
  triangle = FALSE,
  ...
)
}
\arguments{
\item{data}{Optional data vector. If supplied, empirical moments are computed
from \code{data} and \code{stats} is ignored.}

\item{stats}{Optional numeric vector of length 4 containing mean, variance,
skewness, and kurtosis. Entries may be \code{NA} when the corresponding
parameter is fixed.}

\item{mu}{Location parameter, if fixed.}

\item{delta}{Scale parameter, if fixed.}

\item{beta}{Skewness parameter, if fixed.}

\item{alpha}{Tail parameter, if fixed.}

\item{triangle}{Logical; if \code{TRUE}, return only the asymmetry and
steepness summary parameters.}

\item{...}{Additional arguments passed to \code{\link[stats]{optim}}.}
}
\value{
If \code{triangle = FALSE}, a named numeric vector with entries
\code{mu}, \code{delta}, \code{beta}, and \code{alpha}.
If \code{triangle = TRUE}, a named numeric vector with entries
\code{asymmetry} and \code{steepness}.
}
\description{
Estimates Normal-Wald parameters from sample moments or from a supplied
vector of moments.
}
\details{
This function computes method-of-moments estimates for the Normal-Wald
distribution. If \code{data} is supplied, sample mean, variance, skewness,
and kurtosis are computed automatically. Otherwise, the user may provide
these moments directly through \code{stats}.

If some parameters are fixed, only the remaining parameters are estimated.
When \code{triangle = TRUE}, the returned values are the transformed summary
quantities
\eqn{\text{steepness} = (1 + |\gamma|)^{-1/2}} and a corresponding
asymmetry measure.
}
\examples{
n <- 500
y <- rgamma(n, 3, 1.5) + rlnorm(n, 1, 0.5)
z <- (y - mean(y)) / sd(y)
skew <- mean(z^3)
kurt <- mean(z^4)

nw_est_mom(stats = c(NA, NA, skew, kurt), mu = 0, delta = 1, triangle = TRUE)
nw_est_mom(stats = c(NA, var(y), skew, kurt), mu = 0, triangle = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbass.R
\name{plot.gbass}
\alias{plot.gbass}
\title{Plot method for gbass objects}
\usage{
\method{plot}{gbass}(x, ...)
}
\arguments{
\item{x}{A fitted object of class \code{"gbass"}.}

\item{...}{Additional graphical arguments.}
}
\value{
No return value, called for its side effects.
}
\description{
Plot method for gbass objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nw_tools.R
\name{nw_gamma_prior}
\alias{nw_gamma_prior}
\title{Calibrate a prior for the Normal-Wald gamma parameter}
\usage{
nw_gamma_prior(
  q1 = 0.1,
  q2 = 0.9,
  p1 = 0.5,
  p2 = 0.05,
  par0 = NULL,
  lambda = 0
)
}
\arguments{
\item{q1}{Lower reference value for the steepness parameter. Default is
\code{0.1}.}

\item{q2}{Upper reference value for the steepness parameter. Default is
\code{0.9}.}

\item{p1}{Target probability associated with \code{q1}. Default is
\code{0.5}.}

\item{p2}{Target probability associated with \code{q2}. Default is
\code{0.05}.}

\item{par0}{Optional starting values for the numerical optimization.}

\item{lambda}{Optional ridge penalty used in the optimization. Default is
\code{0}.}
}
\value{
A named numeric vector with entries \code{m_gamma} and
\code{s_gamma}.
}
\description{
Selects prior hyperparameters for \code{gamma} in the Normal-Wald model using
probability statements about the steepness parameter
\eqn{\xi = (1 + \gamma)^{-1/2}}.
}
\details{
This function chooses hyperparameters for a prior on \code{gamma} by solving
a simple calibration problem based on the steepness parameter
\eqn{(1 + \gamma)^{-1/2}}. The optimization is carried out with
\code{\link[stats]{optim}} using the Nelder-Mead method.
}
\examples{
nw_gamma_prior()
nw_gamma_prior(q1 = 0.2, q2 = 0.8, p1 = 0.4, p2 = 0.1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbass.R
\name{gbass}
\alias{gbass}
\title{Generalized Bayesian MARS}
\usage{
gbass(
  X,
  y,
  w_prior = list(type = "GIG", p = 0, a = 0, b = 0),
  v_prior = list(type = "GIG", p = -15, a = 0, b = 30),
  maxInt = 3,
  maxBasis = 1000,
  npart = NULL,
  nmcmc = 10000,
  nburn = 9000,
  thin = 1,
  moveProbs = rep(1/3, 3),
  a_tau = 1/2,
  b_tau = NULL,
  a_lambda = 1,
  b_lambda = 1,
  m_beta = 0,
  s_beta = 0,
  scale = 1,
  Iw0 = rep(1, maxInt),
  Zw0 = rep(1, ncol(X)),
  verbose = TRUE
)
}
\arguments{
\item{X}{An \eqn{N \times p} numeric matrix of predictors. A numeric vector is
treated as a single-column matrix.}

\item{y}{A numeric response vector of length \eqn{N}.}

\item{w_prior}{A named list specifying the prior for the global variance
component. See Details.}

\item{v_prior}{A named list specifying the shared prior for the local variance
components. See Details.}

\item{maxInt}{Integer giving the maximum interaction degree in proposed basis
functions.}

\item{maxBasis}{Maximum number of basis functions.}

\item{npart}{Minimum number of nonzero points required for a proposed basis
function. Defaults to \code{min(20, 0.1 * N)}.}

\item{nmcmc}{Total number of MCMC iterations.}

\item{nburn}{Number of initial iterations discarded as burn-in.}

\item{thin}{Thinning interval for retained draws.}

\item{moveProbs}{A length-3 vector giving probabilities for birth, death, and
mutation moves.}

\item{a_tau}{Prior hyperparameter for \code{tau}.}

\item{b_tau}{Prior hyperparameter for \code{tau}. Defaults to \code{N / 2}.}

\item{a_lambda}{Prior hyperparameter for \code{lambda}.}

\item{b_lambda}{Prior hyperparameter for \code{lambda}.}

\item{m_beta}{Prior mean for \code{beta}.}

\item{s_beta}{Prior standard deviation for \code{beta}.}

\item{scale}{Fixed variance-scale parameter. Defaults to \code{1}.}

\item{Iw0}{Vector of positive nominal weights for interaction order in proposed
basis functions. Must have length \code{maxInt}.}

\item{Zw0}{Vector of positive nominal weights for variable selection in
proposed basis functions. Must have length \code{ncol(X)}.}

\item{verbose}{Logical; should progress be printed?}
}
\value{
An object of class \code{"gbass"} containing posterior draws and fitted model
information.
}
\description{
Fits a generalized Bayesian MARS (GBASS) model for nonlinear regression under
flexible latent-scale likelihoods.
}
\details{
The priors for \code{w} and \code{v_i} must belong to the generalized inverse
Gaussian (\code{"GIG"}) or generalized beta prime (\code{"GBP"}) families.

Each prior list may contain:
\enumerate{
\item \code{type}: either \code{"GIG"} or \code{"GBP"}.
\item \code{p}, \code{a}, \code{b}: prior hyperparameters.
\item \code{lower_bound}: optional lower bound for the parameter support.
For backward compatibility, \code{lb} is also accepted.
\item \code{prop_sigma}: proposal standard deviation on the log scale for
Metropolis updates, when applicable.
\item \code{adapt}, \code{adapt_delay}, \code{adapt_thin}: optional controls
for adaptive Metropolis updates when applicable.
}

For \code{w}, \code{prop_sigma} and the adaptive Metropolis fields are mainly
relevant when \code{w_prior$type = "GBP"}. In the \code{"GIG"} case with
nonzero \code{beta}, \code{w} is sampled using the modified half-normal
formulation rather than a random-walk Metropolis step.

Retained draws are taken at iterations
\code{nburn + 1, nburn + 1 + thin, ...}. Thus \code{nburn} is interpreted as
the number of initial iterations discarded as burn-in.
}
\note{
Current implementation notes:
\enumerate{
\item Basis function parameters are stored as lists.
\item Knot locations use continuous uniform proposals.
\item Basis coefficients use a ridge-type prior.
}
}
\examples{
ff1 <- function(x) 10.391*((x[1]-0.4)*(x[2]-0.6) + 0.36)

n <- 100
p <- 4
X <- matrix(runif(n * p), nrow = n)
y <- apply(X, 1, ff1)
mod <- gbass(X, y, nmcmc = 1000, nburn = 900, thin = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GBASS-package.R
\docType{package}
\name{GBASS-package}
\alias{GBASS}
\alias{GBASS-package}
\title{GBASS: Generalized Bayesian Adaptive Smoothing Splines}
\description{
Bayesian nonlinear regression under a range of likelihood models using generalized Bayesian adaptive smoothing splines. Robust regression with Student's t likelihoods, quantile regression, and related latent-scale models are included as special cases.
}
\author{
\strong{Maintainer}: Kellin Rumsey \email{knrumsey@lanl.gov}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol.R
\name{gsobol}
\alias{gsobol}
\title{Sobol decomposition for GBASS models}
\usage{
gsobol(object, ...)
}
\arguments{
\item{object}{A fitted object of class \code{"gbass"}.}

\item{...}{Additional arguments passed to \code{BASS::sobol()}.}
}
\value{
An object of class \code{"bassSob"} returned by \code{BASS::sobol()}.
}
\description{
Computes Sobol sensitivity indices for a fitted \code{gbass} model by first
converting it to a compatible \code{BASS} object with \code{gbass2bass()},
and then calling \code{BASS::sobol()}.
}
\details{
This is a thin wrapper around \code{BASS::sobol()} for GBASS models.
Users who want direct access to the converted \code{BASS} object can call
\code{gbass2bass()} explicitly and then use \code{BASS::sobol()} themselves.
}
\examples{
ff1 <- function(x) 10.391*((x[1]-0.4)*(x[2]-0.6) + 0.36)

n <- 100
p <- 4
X <- matrix(runif(n * p), nrow = n)
y <- apply(X, 1, ff1)

mod <- gbass(X, y, nmcmc = 1000, nburn = 900)

# Direct wrapper
sob <- gsobol(mod)

# Equivalent manual conversion
bm <- gbass2bass(mod)
sob2 <- BASS::sobol(bm)

}
\seealso{
\code{\link{gbass2bass}}, \code{\link[BASS]{sobol}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{gbass2bass}
\alias{gbass2bass}
\alias{gm2bm}
\title{Convert GBASS object to BASS-like object}
\usage{
gbass2bass(gm)

gm2bm(gm)
}
\arguments{
\item{gm}{An object of class \code{gbass}.}
}
\value{
An object with class \code{c("bass", "gbass")}.
}
\description{
Converts an object of class \code{gbass} to an object with class \code{bass},
so that downstream BASS utilities such as Sobol decomposition can be used.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{rgig2}
\alias{rgig2}
\title{Generalized Inverse Gaussian Generator}
\usage{
rgig2(p, a, b)
}
\arguments{
\item{p}{a real valued parameter}

\item{a}{a non-negative parameter. If a=0, then p must be negative.}

\item{b}{a non-negative parameter. If b=0, then p must be positive.}
}
\value{
A numeric vector of random draws from the generalized inverse Gaussian distribution.
}
\description{
This function generates samples from the GIG(p, a, b) distribution with density function
f(x) = cx^(p-1)exp(-1/2*(a*x + b/x))
}
\details{
A uniformly bounded rejection sample based on Hörmann et. al. (2014). Special cases include Gamma (b=0, p>0), Inverse Gamma (a=0, p<0) and Inverse Gaussian (p=-1/2).
}
\examples{
x <- rep(NA, 1000)
for(i in 1:1000) x[i] <- rgig2(2, 3, 0.5)
hist(x)

}
\references{
Hörmann, Wolfgang, and Josef Leydold. "Generating generalized inverse Gaussian random variates." Statistics and Computing 24.4 (2014): 547-557.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers.R
\name{tbass}
\alias{tbass}
\title{TBASS - Bayesian MARS with a Student's t likelihood}
\usage{
tbass(X, y, df = 5, ...)
}
\arguments{
\item{X}{An \eqn{N \times p} numeric matrix of predictor variables.}

\item{y}{A numeric response vector of length \eqn{N}.}

\item{df}{Degrees of freedom. Default is \code{5}.}

\item{...}{Additional arguments passed to \code{gbass()}.}
}
\value{
An object of class \code{c("tbass", "gbass")} containing posterior draws
and fitted model information.
}
\description{
Wrapper around \code{gbass()} for a Student's t error model.
}
\details{
Uses an inverse-gamma latent-scale representation through a GIG prior on the
local variance components. If \code{df > 2}, the scale is set to
\code{(df - 2) / df} so that the marginal variance matches \code{w}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_prior.R
\name{build_prior}
\alias{build_prior}
\alias{build_GIG}
\alias{build_GBP}
\title{Construct a prior specification for GBASS}
\usage{
build_prior(
  type = c("GIG", "GBP"),
  p,
  a,
  b,
  lower_bound = NULL,
  prop_sigma = NULL,
  adapt = NULL,
  adapt_delay = NULL,
  adapt_thin = NULL
)

build_GIG(
  p,
  a,
  b,
  lower_bound = NULL,
  prop_sigma = NULL,
  adapt = NULL,
  adapt_delay = NULL,
  adapt_thin = NULL
)

build_GBP(
  p,
  a,
  b,
  lower_bound = NULL,
  prop_sigma = NULL,
  adapt = NULL,
  adapt_delay = NULL,
  adapt_thin = NULL
)
}
\arguments{
\item{type}{"GIG" or "GBP"}

\item{p, a, b}{prior hyperparameters}

\item{lower_bound}{optional lower bound}

\item{prop_sigma}{optional proposal sd (log scale)}

\item{adapt}{logical; use adaptive MH?}

\item{adapt_delay}{delay before adapting}

\item{adapt_thin}{thinning for adaptation updates}
}
\value{
A named list containing the prior specification.
}
\description{
Construct a prior specification for GBASS
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gbass.R
\name{predict.gbass}
\alias{predict.gbass}
\title{Predict method for GBASS objects}
\usage{
\method{predict}{gbass}(
  object,
  newdata = NULL,
  mcmc.use = NULL,
  predictive = TRUE,
  bias_correct = FALSE,
  samples = 1,
  ...
)
}
\arguments{
\item{object}{an object of class "gbass" (including subclasses like
"tbass", "qbass", "nwbass")}

\item{newdata}{a matrix of predictor variables. Defaults to training inputs.}

\item{mcmc.use}{optional vector indicating which posterior draws to use.}

\item{predictive}{logical. If TRUE, return posterior predictive draws.
If FALSE, return draws of the linear predictor.}

\item{bias_correct}{logical. Ignored unless predictive = FALSE.
If TRUE, return the posterior mean response rather than just the linear predictor.}

\item{samples}{Integer giving the number of predictive samples to generate
per retained MCMC draw when \code{predictive = TRUE}. Ignored when
\code{predictive = FALSE}. Default is \code{1}.}

\item{...}{Additional graphical arguments (not used)}
}
\value{
a matrix with rows corresponding to posterior draws and columns
corresponding to rows of newdata.
}
\description{
Returns posterior draws of either:
(i) the linear predictor / mean surface, or
(ii) the full posterior predictive distribution.
}
\details{
If predictive = FALSE and bias_correct = FALSE, this returns draws of B(x)a.

If predictive = FALSE and bias_correct = TRUE, this returns draws of
B(x)a + E(error | posterior draw),
i.e. the mean response under the fitted GBASS error model.

If predictive = TRUE, this returns posterior predictive draws by simulating
a fresh latent local variance v_new and Gaussian draw for each posterior sample.

For qbass objects, bias_correct = TRUE is usually not what you want, because
qbass is typically being used for quantile regression rather than mean regression.

Currently, posterior predictive draws are implemented for GIG-based models.
If the fitted object uses a GBP prior for v, predictive = TRUE will stop.
}
